/*-----------------------------------------
   MAP.C -- Map procedures module
   Version date: 99.10.22
  -----------------------------------------*/


#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "external.h"

// Prototypes:
void  Map () ;
void  ShowMapWidth() ;
void  ZoomIn () ;
void  ZoomOut () ;

/*-----------------------------------------
   Map: Updates map    
   Version date: 99.10.22
-----------------------------------------*/
void Map() 
  {
  float  fCentreLon, fCentreLat ;
  float  fDist ;
  float  fMapScale ;
  SIZE   size ;
  char   cText[25] ;
  HDC    hdc ;
  int    iI ;
  int    iTrackCount ;

  fMapScale = 1000./fMapWidth ;

  // check for edge of map if map is tracking ship
  if (bTracking && ((fLongtitude > fRightEdge) || (fLongtitude < fLeftEdge) ||
                    (fLatitude > fTopEdge) || (fLatitude < fBottomEdge)))
    bMapMustBePainted = TRUE ;

  if (bMapMustBePainted)
    { 
    if (bTracking)
      {
      fLeftEdge = 0.0 ;
      fRightEdge = fMapWidth ;
      fBottomEdge = 0.0 ;
      fTopEdge = fMapWidth ;
      while (fLongtitude > fRightEdge)
        {
        fLeftEdge = fLeftEdge + fMapRadius ;
        fRightEdge = fRightEdge + fMapRadius ;
        }
      while (fLongtitude < (fLeftEdge+fOneThird))
        {
        fRightEdge = fRightEdge - fMapRadius ;
        fLeftEdge = fLeftEdge - fMapRadius ;
        }
      while (fLatitude > fTopEdge)
        {
        fBottomEdge = fBottomEdge + fMapRadius ;
        fTopEdge = fTopEdge + fMapRadius ;
        }
      while (fLatitude < (fBottomEdge+fOneThird))
        {
        fTopEdge = fTopEdge - fMapRadius ;
        fBottomEdge = fBottomEdge - fMapRadius ;
        }
      }
     
    // show coordinates of north-west and south=east corners
    hdc = GetDC (hwnd) ;
    if (fTopEdge >=0)
      {
      if (fLeftEdge >= 0)
        sprintf(cText, "%4.2f nm N, %4.2f nm E", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      else
        sprintf(cText, "%4.2f nm N, %4.2f nm W", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      }
    else
      {
      if (fLeftEdge >= 0)
        sprintf(cText, "%4.2f nm S, %4.2f nm E", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      else
        sprintf(cText, "%4.2f nm S, %4.2f nm W", fabs(fTopEdge), fabs(fLeftEdge)) ; 
      }
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, 535, 4, 660, 18) ;
    SelectObject(hdc, hSkyBrush) ;
    SetBkMode(hdc, TRANSPARENT) ;
    GetTextExtentPoint32(hdc,cText, strlen(cText),&size) ; 
    Rectangle (hdc, 535, 4, 539 + size.cx, 18) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    TextOut (hdc, 536, 4, cText, strlen(cText)) ;

    if (fBottomEdge >=0)
      {
      if (fRightEdge >= 0)
        sprintf(cText, "%4.2f nm N, %4.2f nm E", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      else
        sprintf(cText, "%4.2f nm N, %4.2f nm W", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      }
    else
      {
      if (fRightEdge >= 0)
        sprintf(cText, "%4.2f nm S, %4.2f nm E", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      else
        sprintf(cText, "%4.2f nm S, %4.2f nm W", fabs(fBottomEdge), fabs(fRightEdge)) ; 
      }
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    SelectObject(hdc, GetStockObject(LTGRAY_BRUSH)) ;
    Rectangle (hdc, 630, 245, 756, 258) ;
    SelectObject(hdc, hSkyBrush) ;
    SetBkMode(hdc, TRANSPARENT) ;
    GetTextExtentPoint32(hdc,cText, strlen(cText),&size) ; 
    Rectangle (hdc, 750-size.cx, 245, 754, 258) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
    TextOut (hdc, 752, 245, cText, strlen(cText)) ;

    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    SetBkMode(hdc, OPAQUE) ;
    SelectObject(hdc, GetStockObject(LTGRAY_BRUSH)) ;
    ReleaseDC (hwnd, hdc) ; 

    // position map
    hdc = GetDC (hwndChild[2]) ;
    SetMapMode (hdc, MM_ANISOTROPIC) ;
    SetWindowExtEx (hdc, 1000, 1000, NULL) ;
    SetViewportExtEx (hdc, iMapPixels, -iMapPixels, NULL) ;
    SetViewportOrgEx (hdc, 0, iMapPixels, NULL) ;
    // (0,0) = bottom left
    SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,RGB(0,56,207))) ; //sea blue
    SelectObject(hdc, hSeaBrush) ;
    Rectangle (hdc, 0, 0, 1000, 1000) ;
    DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN))) ; 

    // draw islands
    fCentreLon = fRightEdge - fMapRadius ;
    fCentreLat = fTopEdge - fMapRadius ;
    SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,RGB(200,200,0))) ;
    if (iModifyMenu == 4)
      {
      // determine which island is selected
      for (iI=0 ; iI<iFields[iModifyMenu] ; iI++)
        if (GetFocus()==hwndPropText[iI])
          iFocus=iI;
      }
    for (iI = 0 ; iI <= 12 ; iI++)
      {
      if (fIsleRadius[iI] > 0.00001)
        {
        if ((iModifyMenu == 4) && (iI == (int) (iFocus)/3))
          {
          // this island is to be modified (draw dark red)
          SelectObject(hdc, hBlownBrush) ;
          }
        else  
          SelectObject(hdc, hLandBrush) ;  // draw green
        fDist = sqrt((fCentreLon - fIsleLong[iI]) * (fCentreLon - fIsleLong[iI]) + 
          (fCentreLat - fIsleLat[iI]) * (fCentreLat - fIsleLat[iI])) ;
        if (fIsleRadius[iI] > (fDist + 2.0 * fMapRadius))
          Rectangle (hdc, fLeftEdge, fBottomEdge, fRightEdge, fTopEdge) ;
        else
          Ellipse (hdc, 
            ((fIsleLong[iI]-fIsleRadius[iI])-fLeftEdge)*fMapScale, 
            ((fIsleLat[iI]+fIsleRadius[iI])-fBottomEdge)*fMapScale, 
            ((fIsleLong[iI]+fIsleRadius[iI])-fLeftEdge)*fMapScale, 
            ((fIsleLat[iI]-fIsleRadius[iI])-fBottomEdge)*fMapScale ) ;
        } 
      }

    //redraw track
    iTrackCount = iMapCounter ;
    if (bMaxTrack)
      { 
      iTrackCount = 2000 ;
      for (iI = iMapCounter + 1 ; iI < 2000 ; iI++)
        SetPixel (hdc, (int) ((fLon[iI]-fLeftEdge)*fMapScale), 
          (int) ((fLat[iI]-fBottomEdge)*fMapScale), RGB(170,170,170)) ;
      for (iI = 0 ; iI < iMapCounter ; iI++)
        SetPixel (hdc, (int) ((fLon[iI]-fLeftEdge)*fMapScale), 
          (int) ((fLat[iI]-fBottomEdge)*fMapScale), RGB(170,170,170)) ;
      }
    else
      {
      for (iI = 0 ; iI < iTrackCount ; iI++)
        SetPixel (hdc, (int) ((fLon[iI]-fLeftEdge)*fMapScale),
          (int) ((fLat[iI]-fBottomEdge)*fMapScale), RGB(170,170,170)) ;
      }
    } 

  if (!bMapMustBePainted)
    {
    // map has not been redrawn
    hdc = GetDC (hwndChild[2]) ;
    SetMapMode (hdc, MM_ANISOTROPIC) ;
    SetWindowExtEx (hdc, 1000, 1000, NULL) ;
    SetViewportExtEx (hdc, iMapPixels, -iMapPixels, NULL) ;
    SetViewportOrgEx (hdc, 0, iMapPixels, NULL) ;
    }
  
  // draw new position
  SetPixel (hdc, (int) ((fLastLong-fLeftEdge)*fMapScale), 
    (int) ((fLastLat-fBottomEdge)*fMapScale), RGB(170,170,170)) ; // old position
  SetPixel (hdc, (int) ((fLongtitude-fLeftEdge)*fMapScale),
    (int) ((fLatitude-fBottomEdge)*fMapScale), RGB(255,255,255)) ; // new position
  ReleaseDC (hwndChild[2], hdc) ;
  fLastLat = fLatitude ;
  fLastLong = fLongtitude ;

  bMapMustBePainted = FALSE ;

  }

/*-----------------------------------------
   ShowMapWidth
   Version date: 99.10.03
-----------------------------------------*/

void ShowMapWidth()
  {
  HDC    hdc ;
  int    xThis, yThis ;

  hdc = GetDC (hwnd) ;
  xThis = 760 ;
  yThis = 115 ;
  if (fMapWidth>10)
    xThis=xThis - 5 ; 
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, 755, yThis, 790, yThis + iTextHeight) ;
  TextOut (hdc, xThis, yThis, sMessage, sprintf(sMessage, "%4.2f", fMapWidth)) ; 
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  ReleaseDC (hwnd, hdc) ;
  }

/*-----------------------------------------
   ZoomIn: Zooms map in    
   Version date: 99.09.16
-----------------------------------------*/

void ZoomIn() 
  {
  
  if (fMapWidth > 0.04)
    {
    fMapWidth = fMapWidth / 5.0 ;
    fMapRadius = fMapWidth / 2.0 ;
    fOneThird = 0.67 * fMapRadius ;
    ShowMapWidth() ;
    bMapMustBePainted = TRUE ;
    if (!bTracking)
      {
      fLeftEdge = fLeftEdge + 2 * fMapWidth ;
      fBottomEdge = fBottomEdge + 2 * fMapWidth ;
      fRightEdge = fLeftEdge + fMapWidth ;
      fTopEdge = fBottomEdge + fMapWidth ;
      }
    }

  EnableWindow (hwndButton[1], TRUE) ; //zoom out enabled
  if (fMapWidth < 0.1)
    EnableWindow (hwndButton[0], FALSE) ; //zoom in disabled
  SetFocus (hwndCheck[2]) ;

  }

/*-----------------------------------------
   ZoomOut: Zooms map out    
   Version date: 99.09.16
-----------------------------------------*/

void ZoomOut() 
  {
  
  if (fMapWidth < 20.0)
    {
    fMapWidth = fMapWidth * 5.0 ;
    fMapRadius = fMapWidth / 2.0 ;
    fOneThird = 0.67 * fMapRadius ;
    ShowMapWidth() ;
    bMapMustBePainted = TRUE ;
    if (!bTracking)
      {
      fLeftEdge = fLeftEdge - 0.4 * fMapWidth ;
      fBottomEdge = fBottomEdge - 0.4 * fMapWidth ;
      fRightEdge = fLeftEdge + fMapWidth ;
      fTopEdge = fBottomEdge + fMapWidth ;
      }
    }

  EnableWindow (hwndButton[0], TRUE) ; //zoom in enabled
  if (fMapWidth > 15.)
    EnableWindow (hwndButton[1], FALSE) ; //zoom out disabled
  SetFocus (hwndCheck[2]) ;

  }
