/*-----------------------------------------
   OTHERS.C -- Other procedures module
   Version date: 04.03.07
  -----------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2004, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#define   CONFACT  0.25   //determines sharpness of stern of ship on PlanView

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "external.h" 
#include "Menu.h" 

// Prototypes:
void   CanShadow() ;
void   CycleEvents() ;
void   DerivedSailParameters() ;
void   DerivedShipParameters() ;
void   SelectSailParameter() ;
int    Sgn(float) ;
void   ShowActualHelmAngle() ;
void   ShowActualJibAngle() ;
void   ShowActualStaysailsAngle() ;
void   ShowActualYardAngle(int) ;
void   ShowSelectedHelmAngle() ;
void   ShowSelectedYardAngle(int) ;
void   YardAdjust (int) ;

extern void   AdjustSails() ;
extern int    Beaufort (float) ;
extern void   EndPlayBack () ;
extern float  Extract (char *, int) ; 
extern void   ManBraces () ;
extern void   ManHelm () ;
extern void   ShowMessage() ;

/* ------------------------------
     CanShadow
     Version date: 03.03.17
-------------------------------*/

void CanShadow()
  {
  // Calculates which sails (as set) are at the right height to
  //  shadow a particular sail

  int    iI, iJ, iN, iS ;
  char   cSl, cSh ;
  BOOL   bShad ;
 
  for (iS = 0; iS < 4 ; iS++)
    {
    for (iI = 0 ; iI < iNrSails ; iI++)
      {
      for (iN = 0; iN < iMaxShad ; iN++)
        iSailCanShad[iS][iI][iN] = -1 ;
      }
    }

  for (iS = 0; iS < 4 ; iS++)
    {
    for (iI = 0 ; iI < (iNrSails-1) ; iI++)
      {
      if (bBrig)
        cSl = iBrigSailSequence[iS][iI] ;
      else
        cSl = iFrigateSailSequence[iS][iI] ;
      if (iSailStatus[cSl] < 4)
        {
        iN = 0 ;
        for (iJ = iI + 1 ; iJ < iNrSails ; iJ++)
          {
          if (bBrig)
            cSh = iBrigSailSequence[iS][iJ] ;
          else
            cSh = iFrigateSailSequence[iS][iJ] ;
          // cSh=shaddowing sail
          bShad = TRUE ;
          if (iSailStatus[cSh] > 3) bShad = FALSE ;
          if (fSailEffectiveHead[cSh] < fSailFoot[cSl]) bShad = FALSE ;
          if (fSailFoot[cSh] > fSailEffectiveHead[cSl]) bShad = FALSE ;
          if ((bShad) && (iN < iMaxShad ))
            {
            iSailCanShad[iS][cSl][iN] = cSh ;
            //if ((iS==1) && (cSl==12))
            //  {
            //  sprintf(sMessage, "Shad %3d: %3d", iN, iSailCanShad[iS][cSl][iN]); 
            //  MessageBox (hwnd, sMessage, "Hello" ,MB_ICONINFORMATION | MB_OK) ;
            //  }
            iN = iN + 1 ;
            }
          }
        }
      }
    }
  }

/* --------------------------------------
   CycleEvents
   Version date = 03.08.13
---------------------------------------*/

void CycleEvents()
  {
  HDC   hdc ;
  char  sBuf [15];
  int   iDelta, iSign ;
  int   iI, iJ, i1, i2 ; 
  int   yThis ;
  float fAngle ;
  float fDistance ;
  float fWindGap ;


  // remove bearing from plan view if approprite 
  if (fNow > (fBearingTime + 1))
    {
    hdc=GetDC (hwnd) ;
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xBearing+1, yBearing, xBearing - 25, yBearing + 12) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    ReleaseDC (hwnd, hdc) ;
    }

  // change wind direction if necessary
  if (bWindDirChange)
    {
    fWindGap = fNewWindDirection - fWindDirection ;
    if (fWindGap > 180.)
      fWindGap = fWindGap - 360.0 ;
    if (fWindGap < -180.0)
      fWindGap = fWindGap + 360.0 ;
    if (fabs(fWindGap) > 0.5)
      {
      if (fWindGap > 0.)
        {
        fWindDirection = fWindDirection + fDeltaWindDir ;
        if (fWindDirection > 359.9)
          fWindDirection = fWindDirection - 360.0 ;
        }
      else
        {
        fWindDirection = fWindDirection - fDeltaWindDir ;
        if (fWindDirection < 0.)
          fWindDirection = fWindDirection + 360.0 ;
        }  
      sprintf(sMessage, "%3d", (int) fWindDirection) ; 
      if (strnicmp (sMessage, sOldWindDirection, 3))
        {
        hdc=GetDC (hwnd) ;
        SetTextColor(hdc, RGB(0,100,0)) ; // olive green
        yThis = yWind + cyTextHeight ;
        SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
        TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "  %3.0f ", fWindDirection)); 
        SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
        strncpy(sOldWindDirection,sMessage,3) ; 
        bPlanViewIsChanged = TRUE ;
        SetTextColor(hdc, RGB(0,0,0)) ; // black
        ReleaseDC (hwnd, hdc) ;
        }
      }      
    else
      {
      bWindDirChange = FALSE ;
      fWindDirection = fNewWindDirection ;
      sprintf(sMessage, "Wind now steady from %3.0f ", fWindDirection); 
      ShowMessage () ;
      }
    }

  // change wind speed if necessary
  if (bWindSpeedChange)
    {
    if (fNewWindSpeed > fWindSpeed)
      fWindSpeed = fWindSpeed + fDeltaWindSpeed ;
    else
      fWindSpeed = fWindSpeed - fDeltaWindSpeed ;
    sprintf(sMessage, "%3d", (int) fWindSpeed) ; 
    if (strnicmp (sMessage, sOldWindSpeed, 3))
      {
      hdc=GetDC (hwnd) ;
      SetTextColor(hdc, RGB(0,100,0)) ; // olive green
      yThis = yWind + cyTextHeight ;
      TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, " at %3.0f kt (%d Bf)    ", fWindSpeed, Beaufort(fWindSpeed))); 
      SetTextColor(hdc, RGB(0,0,0)) ; // black
      ReleaseDC (hwnd, hdc) ;
      strncpy(sOldWindSpeed,sMessage,3) ; 
      bPlanViewIsChanged = TRUE ;
      }
    if (fabs(fNewWindSpeed-fWindSpeed)< 0.5) 
      {
      bWindSpeedChange = FALSE ;
      fWindSpeed = fNewWindSpeed ;
      sprintf(sMessage, "Wind now steady at %3.0f kts", fWindSpeed); 
      ShowMessage () ;
      }
    }
  
  // Adjust Planview if on AutoBearing
  if (bAutoBearing)
    {
    fOffsetAngle = fHeading ;
    sprintf(sMessage, "%3d", (int) fOffsetAngle) ;
    SetWindowText(hwndText, sMessage) ;
    bPlanViewIsChanged = TRUE ;
    }

  // Automatic Crewmen
  if (bYardsmen)
    ManBraces() ;
  if (bHelmsman && iTackStatus==0)
    ManHelm() ;

  // change helm if necessary
  if (iHelmAngle != iNewHelmAngle)
    {
    iDelta = iNewHelmAngle - iHelmAngle ;
    bPlanViewIsChanged = TRUE ;
    if (abs(iDelta) <= 5)
      iHelmAngle = iHelmAngle + Sgn((float) iDelta) ;
    else
      iHelmAngle = iHelmAngle + Sgn((float) iDelta) * 5 ;
    ShowActualHelmAngle() ;
    } 

  // change yard positions if necessary
  for (iI = 0 ; iI <= 3 ; iI++)
    {
    iDelta = iNewYardAngle[iI] - iYardAngle[iI] ;
    if (iDelta != 0)
      {
      if (abs(iDelta) <= 5)
        iYardAngle[iI] = iYardAngle[iI] + Sgn((float) iDelta) ;
      else
        iYardAngle[iI] = iYardAngle[iI] + Sgn((float) iDelta) * 5 ;
      switch (iI)
        {
        case 0 :
          for (iJ=0 ; iJ <iSpanker ; iJ++)
            {
            if (iSailType[iJ]==2)
              fSailAngle[iJ] = iYardAngle[0] ;
            }
          break ;
        case 1 :
          for (iJ=0 ; iJ <iSpanker ; iJ++)
            {
            if (iSailType[iJ]==4)
              fSailAngle[iJ] = iYardAngle[1] ;
            }
          break ;
        case 2 :
          for (iJ=0 ; iJ <iSpanker ; iJ++)
            {
            if (iSailType[iJ]==6)
              fSailAngle[iJ] = iYardAngle[2] ;
            }
          break ;
        case 3 :
          fSailAngle[iSpanker] = iYardAngle[3] ;
        }
      bPlanViewIsChanged = TRUE ;
      ShowActualYardAngle(iI) ;
      }
    }

  // scroll message display if necessary
  if (iMessageNr > -1)
    {
    if (fNow >= fMessageTime[0]+10.)
      {
      hdc=GetDC (hwnd) ;
      for (iI = 0; iI <= 4; iI++)
        {
        strcpy(sShipMessage[iI],sShipMessage[iI+1]) ;
        fMessageTime[iI] = fMessageTime[iI+1] ;
        SelectObject (hdc,GetStockObject(NULL_PEN)) ;
        Rectangle (hdc, xMessage, yMessage + iI*cyTextHeight, xMessage+cxMessage-2, yMessage + (iI+1)*cyTextHeight+2) ;
        SelectObject (hdc,GetStockObject(BLACK_PEN)) ;
        TextOut (hdc, xMessage, yMessage + iI*cyTextHeight, sShipMessage[iI], strlen(sShipMessage[iI])) ; 
        }
      sShipMessage[5][0]='\0' ;
      fMessageTime[5] = 999999. ;
      SelectObject (hdc,GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xMessage, yMessage+ 5*cyTextHeight, xMessage+cxMessage-2, yMessage + 6*cyTextHeight+2) ;
      SelectObject (hdc,GetStockObject(BLACK_PEN)) ;
      ReleaseDC (hwnd, hdc) ;
      iMessageNr-- ;
      }
    }

  // adjust sails if necessary
  if (iClicks > -1)
    AdjustSails();

  // update position on map
  iMapSkip = iMapSkip + 1 ;
  if (iMapSkip == iMapSkipFactor)
    {
    iMapSkip = 0 ;
    fLat[iMapCounter] = fLatitude ;
    fLon[iMapCounter] = fLongitude ;
    iMapCounter = iMapCounter + 1 ;
    if (iMapCounter >= 2000)
      {
      for (iI = 1 ; iI < 1000 ; iI++)
        {
        fLat[iI] = fLat[iI*2] ;
        fLon[iI] = fLon[iI*2] ;
        }  
      iMapCounter = 1000 ;
      iMapSkipFactor = iMapSkipFactor*2 ;
      }
    }

  // Sound buzzer if on land
  for (iI = 0 ; iI <= 12 ; iI++)
    {
    fDistance = sqrt((fIsleLat[iI] - fLatitude) * (fIsleLat[iI] - fLatitude) + (fIsleLong[iI] - fLongitude) * (fIsleLong[iI] - fLongitude)) ;
    if (fDistance < fIsleRadius[iI])
      MessageBeep(0) ;
    }

  // Show number of update cycles
  hdc=GetDC (hwnd) ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xLeft, yBottomLine, xLeft + 20, yBottomLine + cyTextHeight-1) ;
  SetTextColor(hdc, RGB(150,150,150)) ; // dark grey
  TextOut (hdc, xLeft, yBottomLine, sMessage, sprintf (sMessage, "%1d", (2*iCycles))); 
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  ReleaseDC (hwnd, hdc) ;
  iCycles = 0 ;

  }    

/* --------------------------------------
   DerivedSailParameters
   Version date = 03.09.17
----------------------------------------*/

void DerivedSailParameters()
  {
  int    iI, iJ, i2 ;
  float  fTemp ;

  // Sail names and types; max sail name length = 24!
  if (bBrig)
    {
    sSailName[0] = "Fore Royal" ;
    sSailName[1] = "Fore Topgallant" ;
    sSailName[2] = "Fore Upper Topsail" ;
    sSailName[3] = "Fore Lower Topsail" ;
    sSailName[4] = "Fore Course" ;
    sSailName[5] = "Main Royal" ;
    sSailName[6] = "Main Topgallant" ;
    sSailName[7] = "Main Upper Topsail" ;
    sSailName[8] = "Main Lower Topsail" ;
    sSailName[9] = "Main Course" ;
    sSailName[10] = "Spanker" ;
    sSailName[11] = "Flying Jib" ;
    sSailName[12] = "Outer Jib" ;
    sSailName[13] = "Inner Jib" ;
    sSailName[14] = "Fore Topmast Staysail" ;
    sSailName[15] = "Main Royal Staysail" ;
    sSailName[16] = "Main Topgallant Staysail" ;
    sSailName[17] = "Main Topmast Staysail" ;
    iSailType[0]= 2 ; // foremast sail
    iSailType[1]= 2 ;
    iSailType[2]= 2 ;
    iSailType[3]= 2 ;
    iSailType[4]= 2 ;
    iSailType[5]= 4 ; // mainmast sail
    iSailType[6]= 4 ;
    iSailType[7]= 4 ;
    iSailType[8]= 4 ;
    iSailType[9]= 4 ;
    iSailType[10]= 4 ;  
    iSailType[11]= 1 ; // foremast staysail
    iSailType[12]= 1 ;
    iSailType[13]= 1 ;
    iSailType[14]= 1 ;
    iSailType[15]= 3 ;  // mainmast staysail
    iSailType[16]= 3 ;
    iSailType[17]= 3 ;
    }
  else
    {
    sSailName[0] = "Fore Royal" ;
    sSailName[1] = "Fore Topgallant" ;
    sSailName[2] = "Fore Topsail" ;
    sSailName[3] = "Fore Course" ;
    sSailName[4] = "Main Royal" ;
    sSailName[5] = "Main Topgallant" ;
    sSailName[6] = "Main Topsail" ;
    sSailName[7] = "Main Course" ;
    sSailName[8] = "Mizzen Royal" ;
    sSailName[9] = "Mizzen Topgallant" ;
    sSailName[10] = "Mizzen Topsail" ;
    sSailName[11] = "Spanker" ;
    sSailName[12] = "Jib" ;
    sSailName[13] = "Large jib" ;
    sSailName[14] = "Working jib" ;
    sSailName[15] = "Storm jib" ;
    iSailType[0]= 2 ; // foremast sail
    iSailType[1]= 2 ;
    iSailType[2]= 2 ;
    iSailType[3]= 2 ;
    iSailType[4]= 4 ; // mainmast sail
    iSailType[5]= 4 ;
    iSailType[6]= 4 ;
    iSailType[7]= 4 ;
    iSailType[8]= 6 ; // mizzenmast sail
    iSailType[9]= 6 ;
    iSailType[10]= 6 ;
    iSailType[11]= 6 ;
    iSailType[12]= 1 ; // foremast staysail
    }

  for (iI=0 ; iI<iNrSails ; iI++)
    {
    if (iSailType[iI] == 2)
      {  
      iYardAngle[0] = fSailAngle[iI] ;
      break ;
      }
    }
  for (iI=0 ; iI<iNrSails ; iI++)
    {
    if (iSailType[iI] == 4)
      {  
      iYardAngle[1] = fSailAngle[iI] ;
      break ;
      }
    }
  for (iI=0 ; iI<iNrSails ; iI++)
    {
    if (iSailType[iI] == 6)
      {  
      iYardAngle[2] = fSailAngle[iI] ;
      break ;
      }
    }

  for (iI=0 ; iI<iNrSails ; iI++)
    {
    if (iSailType[iI] == 1)
      {  
      iFirstForeStaySail = iI ;
      break ;
      }
    }

  for (iI=0 ; iI<iNrSails ; iI++)
    {
    if (iSailType[iI] == 3)
      {  
      iFirstMainStaySail = iI ;
      break ;
      }
    }

  if (bBrig)
    i2=2 ;
  else
    i2=3 ;
  for (iI=0 ; iI<i2 ; iI++)
    {
    iNewYardAngle[iI] = iYardAngle[iI] ;
    ShowActualYardAngle(iI) ;
    SetScrollRange (hwndSlider[iI], SB_CTL, -iMaxYardAngle[iI], iMaxYardAngle[iI], FALSE) ;
    SetScrollPos (hwndSlider[iI], SB_CTL, iYardAngle[iI], TRUE) ;
    }
  iYardAngle[3] = fSailAngle[iSpanker] ; // spanker
  iNewYardAngle[3] = iYardAngle[3] ;
  ShowActualYardAngle(3) ;
  SetScrollRange (hwndSlider[3], SB_CTL, -iMaxYardAngle[3], iMaxYardAngle[3], FALSE) ;
  SetScrollPos (hwndSlider[3], SB_CTL, -iYardAngle[3], TRUE) ;
  iNewHelmAngle = iHelmAngle ;
  ShowActualHelmAngle() ;
  SetScrollRange (hwndSlider[4], SB_CTL, -iMaxHelmAngle, iMaxHelmAngle, FALSE) ;
  if (bModernHelm)
    SetScrollPos (hwndSlider[4], SB_CTL, -iHelmAngle, TRUE) ;
  else
    SetScrollPos (hwndSlider[4], SB_CTL, iHelmAngle, TRUE) ;
  SetScrollRange (hwndSlider[5], SB_CTL, 0, iMaxJibAngle, FALSE) ;
  SetScrollPos (hwndSlider[5], SB_CTL, (int) fSailAngle[iFirstForeStaySail], TRUE) ;
  if (bBrig)
    {
    SetScrollRange (hwndSlider[6], SB_CTL, 0, iMaxJibAngle, FALSE) ;
    SetScrollPos (hwndSlider[6], SB_CTL, (int) fSailAngle[iFirstMainStaySail], TRUE) ;
    }

  // Calculate heights of head and foot of square sails above centre of rotation, and mast heights
  fMastHeight[0] = fLowestSailFoot[0] ; // fore mast
  fMastHeight[1] = fLowestSailFoot[1] ; // main mast
  fMastHeight[2] = fLowestSailFoot[2] ; // mizzen mast

  fTemp = fLowestSailFoot[0] ; // fore mast
  for (iI = iNrSails-1 ; iI >= 0 ; iI--)
    {
    if (iSailType[iI] == 2)
      {
      fSailFoot[iI] = fTemp ;
      fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;
      fMastHeight[0] = fMastHeight[0] + fSailDepth[iI] + 3.0 ;
      fTemp = fTemp + fSailDepth[iI] + 3.0 ;
      }
    }

  fTemp = fLowestSailFoot[1] ; // main mast
  for (iI = iNrSails-1 ; iI >= 0 ; iI--)
    {
    if ((iSailType[iI] == 4) && (iI!= iSpanker))
      {
      fSailFoot[iI] = fTemp ;
      fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;
      fMastHeight[1] = fMastHeight[1] + fSailDepth[iI] + 3.0 ;
      fTemp = fTemp + fSailDepth[iI] + 3.0 ;
      }
    }

  fTemp = fLowestSailFoot[2] ; // mizzen mast
  for (iI = iNrSails-1 ; iI >= 0 ; iI--)
    {
    if ((iSailType[iI] == 6) && (iI!=iSpanker))
      {
      fSailFoot[iI] = fTemp ;
      fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;
      fMastHeight[2] = fMastHeight[2] + fSailDepth[iI] + 3.0 ;
      fTemp = fTemp + fSailDepth[iI] + 3.0 ;
      }
    }

  // spanker
  fTemp = fLowestSailFoot[3] ; 
  iI = iSpanker ;
  fSailFoot[iI] = fLowestSailFoot[3] ;
  fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;

  // Highest mast
  fHighest = fMastHeight[0] ;
  if (fMastHeight[1] > fHighest)
    fHighest = fMastHeight[1] ;
  if (fMastHeight[2] > fHighest)
    fHighest = fMastHeight[2] ;
  if (fMastHeight[3] > fHighest)
    fHighest = fMastHeight[3] ;

  // Calculate positions of corners of sail relative to mast (square sails and spanker)
  // or to vertical plane of tack (staysails); also set fSailCorner (elevation of corner of sail)

  for (iI=0 ; iI < iSpanker ; iI++) // square sails
    {
    xSailPlanHeadLeft[iI] = -fSailHeadWidth[iI] / 2.0 ;
    xSailPlanHeadRight[iI] = fSailHeadWidth[iI] / 2.0 ;
    xSailPlanFootLeft[iI] = -fSailFootWidth[iI] / 2.0 ;
    xSailPlanFootRight[iI] = fSailFootWidth[iI] / 2.0 ;
    ySailPlanHeadLeft[iI] = fSailHead[iI] ;
    ySailPlanHeadRight[iI] = fSailHead[iI] ;
    ySailPlanFootLeft[iI] = fSailFoot[iI] ;
    ySailPlanFootRight[iI] = fSailFoot[iI] ;
    fSailCorner[iI] = fSailFoot[iI] ;
    }

  iI = iSpanker ;  // spanker
  xSailPlanHeadRight[iI] = -fSailHeadWidth[iI] - 1.0 ;
  xSailPlanHeadLeft[iI] = -1.0 ;
  xSailPlanFootRight[iI] = xSailPlanHeadRight[iI] ;
  xSailPlanFootLeft[iI] = xSailPlanHeadLeft[iI] ;
  ySailPlanHeadRight[iI] = fSailHead[iI] + fSpankerPeak ;
  ySailPlanHeadLeft[iI] = fSailHead[iI] ;
  ySailPlanFootRight[iI] = fSailFoot[iI] ;
  ySailPlanFootLeft[iI] = fSailFoot[iI] ;
  fSailCorner[iI] = fSailHead[iI] ;
  
for (iI=iSpanker+1 ; iI < iNrSails ; iI++) // staysails
    {
    iJ = iI-(iSpanker+1) ;
    xSailPlanHeadLeft[iI] = -(fSailTackDistance[iJ]-fSailHeadDistance[iJ]) ;
    xSailPlanHeadRight[iI] = -(fSailTackDistance[iJ]-fSailHeadDistance[iJ]) ;
    xSailPlanFootLeft[iI] = -(fSailTackDistance[iJ]-fSailClewDistance[iJ]) ;
    xSailPlanFootRight[iI] = 0 ;
    ySailPlanHeadLeft[iI] = fSailHeadElevation[iJ] ;
    ySailPlanHeadRight[iI] = fSailHeadElevation[iJ] ;
    ySailPlanFootLeft[iI] = fSailClewElevation[iJ] ;
    ySailPlanFootRight[iI] = fSailTackElevation[iJ] ;
    fSailCorner[iI] = fSailClewElevation[iJ] ;
    }

  // Calculate effective head and mean heights of square sails (and spanker)
  for (iI = 0 ; iI <= iSpanker ; iI++)
    {
    fSailEffectiveHead[iI] = fSailFoot[iI] + fSailDepth[iI] * fSetFraction[iSailStatus[iI]] ;
    fSailMeanHeight[iI] = (fSailEffectiveHead[iI] + fSailFoot[iI]) / 2.0 ;
    if (iI == iSpanker)
      {
      // allow for difference between peak and nock of spanker
      fSailEffectiveHead[iI]=fSailEffectiveHead[iI] + fSpankerPeak ;
      fSailMeanHeight[iI] = ((fSailEffectiveHead[iI]-fSpankerPeak/2) + fSailFoot[iI]) / 2 ;
      } 
   } 

  // Set head and calculate mean height of staysails
 for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
   {
   iJ = iI-(iSpanker+1) ;
   fSailEffectiveHead[iI] = fSailHeadElevation[iJ] ;
   fSailFoot[iI] = fSailTackElevation[iJ] ;
   fSailMeanHeight[iI] = (fSailTackElevation[iJ]+fSailClewElevation[iJ]+fSailHeadElevation[iJ])/3.0;
   fSailSSCentroid[iJ] = (fSailTackDistance[iJ]+fSailClewDistance[iJ]+fSailHeadDistance[iJ])/3.0;
   } 

  // Randomized actual maximum winds speeds for sails
  for (iI = 0 ; iI < iNrSails ; iI++)
    fSailMaxSpeed[iI] = fSailTheorMaxSpeed[iI] * (0.9 + 0.2 * (float) rand()/RAND_MAX) ;

  // Set default values
  for (iI = 0 ; iI < iNrSails ; iI++)
    {
    fSailGradientLeft[iI] = 0.0 ;
    fSailGradientRight[iI] = 0.0 ;
    fSailGradient2[iI] = 0.0 ;
    fSailInterceptLeft[iI] = 0.0 ;
    fSailInterceptRight[iI] = 0.0 ;
    fSailIntercept2[iI] = 0.0 ;
    } 

  // Staysails cannot have reefs
  for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
    iSailMaxReefs[iI] = 0 ;

  CanShadow() ;
  }

/* --------------------------------------
   DerivedShipParameters
   Version date = 04.03.04
----------------------------------------*/

void DerivedShipParameters()
  {
  int   iI, i2 ;
  
  // Dynamic parameters
  fCSABow = fShipWidth * fShipDraught * fBowFactor ; // Cross Sectional Area
  fCSAStern = fShipWidth * fShipDraught * fSternFactor ; 
  fCSASide = fShipLength * fShipDraught * fSideFactor ;
  fShipMass = fShipTons * fMassFactor ;
  fRotationalInertia = fShipMass * fRotatInertFactor * 
    (fShipLength * fShipLength + 3.0 * fShipWidth * fShipWidth / 4.0) / 12.0 ; // cylinder
  fVerticalInertia = fHeelInertFactor * (fShipMass / 2.0) * (fShipWidth / 2.0) * 
     (fShipWidth / 2.0) ;
  fHeelResistFactor = fShipLength * fHeelDragFactor ;
  fMaxHeelMoment = fShipWidth / 4.0 ;

  fHullSide = (fQDeckLength * fQDeckHeight) + (fFocleLength * fQDeckHeight) +
    (fShipLength - (fQDeckLength + fFocleLength)) * fGunDeckHeight ;
  if (fQDeckHeight > fGunDeckHeight)
    fHullEnd = fQDeckHeight * fShipWidth ;
  else
    fHullEnd = fGunDeckHeight * fShipWidth ;

  // scaling of plan view and sailplan
  fMaxFore = fShipLength/2.0 ;
  for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
    {
    if (fSailTackDistance[iI-(iSpanker+1)] > fMaxFore)
      fMaxFore = fSailTackDistance[iI-(iSpanker+1)] ;
    }
  fMaxAft = -fShipLength/2.0 ;
  for (iI = iSpanker+1 ; iI < iNrSails ; iI++)
    {
    if (fSailHeadDistance[iI-(iSpanker+1)] < fMaxAft)
      fMaxAft = fSailHeadDistance[iI-(iSpanker+1)] ;
    }
  if (bBrig)
    iI = 1;
  else
    iI = 2;
  if ((fMastDistance[iI]-fSailHeadWidth[iSpanker]) < fMaxAft)
    fMaxAft = fMastDistance[iI]-fSailHeadWidth[iSpanker] ;
  fScale= max(fabs(fMaxAft),fMaxFore); 
  fScale = fScale + 10 ;

//sprintf(sMessage, "fScew %3.3f ", fScew) ; 
//MessageBox (hwnd, sMessage, "Hello", MB_ICONEXCLAMATION | MB_OK) ;

  for (iI = 0 ; iI <= 7 ; iI++)
    { 
    pHull[iI].x = xHullTemplate[iI] * 0.5 * fShipWidth / fScale ;  
    pHull[iI].y = yHullTemplate[iI] * 0.5 * fShipLength / fScale ;  
    }
  fScale = 1000.0 / fScale ;

  //hull on stern view
  pStern[0].x= -0.4 * fShipWidth ;
  pStern[0].y= fQDeckHeight ;
  pStern[1].x= -fShipWidth ;
  pStern[1].y= -fShipDraught*1.6 ;
  pStern[2].x= fShipWidth ;
  pStern[2].y= -fShipDraught*1.6 ;
  pStern[3].x= 0.4 * fShipWidth ;
  pStern[3].y= fQDeckHeight ;

  }

/* --------------------------------------
   SelectSailParameter
   Version date = 03.08.14
----------------------------------------*/

void SelectSailParameter()
  {
  HDC hdc ;
  int yThis ;
  hdc = GetDC (hwnd) ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xParameter+iSpacer,yFirst+iSpacer,xParameter+iParameterWidth-2*iSpacer,yParameter+cyTextHeight+iSpacer) ;
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  TextOut (hdc, xParameter+iSpacer, yFirst+iSpacer, sParam[iDisplayParam] , 
    strlen(sParam[iDisplayParam])) ; 
  iLastValueToShow = iNrSails-1 ;
  if (iDisplayParam > 2)
    iLastValueToShow = iNrSails+1 ;
  if (iDisplayParam > 5)
    iLastValueToShow = iNrSails+4 ;
  if (iDisplayParam == 8)
    iLastValueToShow = iNrSails+5 ;
  if (iDisplayParam > 5 )
    {
    yThis=yMessage ;
    TextOut (hdc, xTotals, yThis, "Total Thrust",12) ;
    yThis = yThis + cyTextHeight + iSpacer ;
    TextOut (hdc, xTotals, yThis, "Total Drag",10) ;
    yThis = yThis + cyTextHeight + iSpacer ;
    TextOut (hdc, xTotals, yThis, "Resultant",9) ;
    }
  else
    {
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xTotals, yMessage+iSpacer, xTotals+iTotalsWidth-2*iSpacer, yMessage+iTotalsHeight-2*iSpacer) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    }
  if (iDisplayParam == 8)
    {
    SelectObject(hdc, (HPEN) hGreenPen) ;
    Rectangle (hdc, xParameter+iParameterWidth-iHeelParamWidth-2*iSpacer,yFirst+2*iSpacer, xParameter+iParameterWidth-2*iSpacer, yFirst+2*iSpacer+iHeelParamHeight);
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    TextOut (hdc, xParameter+iParameterWidth-iHeelParamWidth,yFirst+3*iSpacer, "Effect of Heel",14) ;
    }
  else
    {
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xParameter+iParameterWidth-iHeelParamWidth-2*iSpacer,yFirst+2*iSpacer, xParameter+iParameterWidth-iSpacer, yFirst+2*iSpacer+iHeelParamHeight);
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    } 
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   Sgn: extracts sign of argument (1=+; 0=0; -1=-)
   Version date = 03.02.09
----------------------------------------*/

int Sgn (float fNumber)
  {
  int    iSign ;
  iSign = 0 ;
  if (fNumber < 0.0)
    iSign = -1 ;
  if (fNumber > 0.0)
    iSign = 1 ;
  return iSign;
  }

/* --------------------------------------
   ShowActualHelmAngle
   Version date = 03.07.22
----------------------------------------*/

void ShowActualHelmAngle ()
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[4] ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xActual, yThis , xActual + 55, yThis + cyTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  switch (Sgn(iHelmAngle))
    {
    case 1 :
      if (bModernHelm)
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iHelmAngle))) ; 
      else
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iHelmAngle))) ; 
      break ;
    case -1 :
      if (bModernHelm)
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iHelmAngle))) ; 
      else
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iHelmAngle))) ; 
      break ;
    default :
      TextOut (hdc, xActual, yThis, "amidships",9) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  bPlanViewIsChanged = TRUE ;
  }

/* --------------------------------------
   ShowActualJibAngle
   Version date = 03.09.11
----------------------------------------*/
void ShowActualJibAngle ()
  {
  HDC   hdc;

  if (iSailStatus[iFirstForeStaySail] != 4) 
    SetScrollPos (hwndSlider[5], SB_CTL, (int) fabs(fSailAngle[iFirstForeStaySail]), TRUE) ;
  hdc = GetDC (hwnd) ;
  SelectObject (hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xJibValue, yJibValue, xJibValue-10*iSpacer, yJibValue+cyTextHeight) ;
  SetTextAlign (hdc, TA_RIGHT|TA_TOP) ;
  if ((int) fabs(fSailAngle[iFirstForeStaySail]) ==45)
    strcpy (sMessage," -E- ") ;
  else
    sprintf (sMessage, "%3d ", (int) fabs(fSailAngle[iFirstForeStaySail])) ; 
  TextOut (hdc, xJibValue, yJibValue, sMessage, 5) ; 
  SetTextAlign (hdc, TA_LEFT|TA_TOP) ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  ReleaseDC (hwnd, hdc) ;
  bPlanViewIsChanged = TRUE ;
  }

/* --------------------------------------
   ShowActualStaysailsAngle
   Version date = 03.09.11
----------------------------------------*/
void ShowActualStaysailsAngle ()
  {
  HDC   hdc;

  if (iSailStatus[iFirstMainStaySail] != 4) 
    SetScrollPos (hwndSlider[6], SB_CTL, (int) fabs(fSailAngle[iFirstMainStaySail]), TRUE) ;
  hdc = GetDC (hwnd) ;
  SelectObject (hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xJibValue, yStaysValue, xJibValue-10*iSpacer, yStaysValue+cyTextHeight) ;
  SetTextAlign (hdc, TA_RIGHT|TA_TOP) ;
  if ((int) fabs(fSailAngle[iFirstMainStaySail]) ==45)
    strcpy (sMessage," -E- ") ;
  else
    sprintf (sMessage, "%3d ", (int) fabs(fSailAngle[iFirstMainStaySail])) ; 
  TextOut (hdc, xJibValue, yStaysValue, sMessage, 5) ; 
  SetTextAlign (hdc, TA_LEFT|TA_TOP) ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  ReleaseDC (hwnd, hdc) ;
  bPlanViewIsChanged = TRUE ;
  }

/* --------------------------------------
   ShowActualYardAngle
   Version date = 03.07.22
----------------------------------------*/

void ShowActualYardAngle (int iI)
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[iI] ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xActual, yThis , xActual + 55, yThis + cyTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  if (iI==3)
    // spanker boom
    {
    switch (Sgn(iYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xActual, yThis, "amidships",9) ; 
      }
    }
  else
  // yards
    {
    switch (Sgn(iYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xActual, yThis, "amidships",9) ; 
      }
    }
  ReleaseDC (hwnd, hdc) ;
  bPlanViewIsChanged = TRUE ;
  }

/* --------------------------------------
   ShowSelectedHelmAngle
   Version date = 03.07.22
----------------------------------------*/

void ShowSelectedHelmAngle ()
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[4] ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xSelected, yThis , xSelected + 55, yThis + cyTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  switch (Sgn(iNewHelmAngle))
    {
    case 1 :
      if (bModernHelm)
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewHelmAngle))) ; 
      else
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iNewHelmAngle))) ; 
      break ;
    case -1 :
      if (bModernHelm)
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iNewHelmAngle))) ; 
      else
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewHelmAngle))) ; 
      break ;
    default :
      TextOut (hdc, xSelected, yThis, "amidships",9) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   ShowSelectedYardAngle
   Version date = 03.07.22
----------------------------------------*/

void ShowSelectedYardAngle (int iI)
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[iI] ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xSelected, yThis , xSelected + 55, yThis + cyTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  if (iI==3)
    //spanker boom
    {
    switch (Sgn(iNewYardAngle[iI]))
      {
      case 1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
        break ;
      case -1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iNewYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xSelected, yThis, "amidships",9) ; 
      }
    }
  else
    //yards
    {
    switch (Sgn(iNewYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iNewYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xSelected, yThis, "amidships",9) ; 
      }
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   YardAdjust
   Version date = 03.09.07
----------------------------------------*/

void YardAdjust(int iIndex)
  {
  int      i1, i2, iI ;

  i1 = iIndex ;
  i2 = iIndex ;
  //check if yards are coupled
  if (bCoupleFore && (iIndex == 0 || iIndex == 1))
    {
    i1 = 0 ;
    i2 = 1 ;
    }
  if (bCoupleMizzen && (iIndex == 1 || iIndex == 2))
    {
    i1 = 1 ;
    i2 = 2 ;
    }
  if (bCoupleFore && bCoupleMizzen && (iIndex != 3))
    {
    i1 = 0 ;
    i2 = 2 ;
    }
  //adjust yard(s)
  for (iI=i1 ; iI<=i2 ; iI++)
    {
    SetScrollPos (hwndSlider[iI], SB_CTL, GetScrollPos (hwndSlider[iIndex], SB_CTL), TRUE) ;
    if (iI==3)
      iNewYardAngle[iI] = -GetScrollPos (hwndSlider[iIndex], SB_CTL) ;
    else
      iNewYardAngle[iI] = GetScrollPos (hwndSlider[iIndex], SB_CTL) ;
    ShowSelectedYardAngle (iI) ;
    } 
  }


