/*-----------------------------------------
   OTHERS.C -- Other procedures module
   Version date: 02.12.14
  -----------------------------------------*/

#define   PI       3.1415927
#define   CONFACT  0.25   //determines sharpness of stern of ship on PlanView

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include "external.h" 
#include "menu.h" 

// Prototypes:
void   CanShadow() ;
void   CycleEvents() ;
void   DerivedSailParameters() ;
void   DerivedShipParameters() ;
void   HelmAdjust() ;
int    Sgn(float) ;
void   ShowActualHelmAngle() ;
void   ShowActualJibAngle() ;
void   ShowActualYardAngle(int) ;
void   SelectSailParameter() ;
void   ShowSelectedHelmAngle() ;
void   ShowSelectedYardAngle(int) ;
void   YardAdjust (int) ;

extern void   AdjustSails() ;
extern void   Calculate() ;
extern void   JibSelect() ; ;
extern void   ManBraces () ;
extern void   ManHelm () ;
extern void   ShowMessage() ;

/* ------------------------------
     CanShadow
     Version date: 99.09.12
-------------------------------*/

void CanShadow()
  {
  // Calculates which sails (as set) are at the right height to
  //  shadow other sails

  int    iI, iJ, iN, iS ;
  char   cSl, cSh ;
  BOOL   bShad ;

  for (iS = 0; iS < 4 ; iS++)
    {
    for (iI = 0 ; iI < 13 ; iI++)
      {
      for (iN = 0; iN < 8 ; iN++)
        iCanShad[iS][iI][iN] = -1 ;
      }
    }

  for (iS = 0; iS < 4 ; iS++)
    {
    for (iI = 0 ; iI < 13 ; iI++)
      {
      cSl = iSequence[iS][iI] ;
      if (iSailStatus[cSl] < 4)
        {
        iN = 0 ;
        for (iJ = iI + 1 ; iJ < 13 ; iJ++)
          {
          cSh = iSequence[iS][iJ] ;
          // cSh=shaddowing sail
          bShad = TRUE ;
          if (iSailStatus[cSh] > 3) bShad = FALSE ;
          if (fEffSailHead[cSh] < fSailFoot[cSl]) bShad = FALSE ;
          if (fSailFoot[cSh] > fEffSailHead[cSl]) bShad = FALSE ;
          if ((bShad) && (iN < 8 ))
            {
            iCanShad[iS][cSl][iN] = cSh ;
            iN = iN + 1 ;
            }
          }
        }
      }
    }
  }

/* --------------------------------------
   CycleEvents
   Version date = 02.12.14
----------------------------------------*/

void CycleEvents()
  {
  HDC   hdc ;
  int   iDelta ;
  int   iI ; 
  int   yThis ;
  float fDistance ;
  float fWindGap ;
  HMENU hMenu;

  //check menu items because they cannot initialised in (e.g.) DerivedShipParameters
  hMenu=GetMenu(hwnd) ;
  if (bShowSlipStream)
    {
    CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_CHECKED) ;
    CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_UNCHECKED) ;
    } 
  else
    { 
    CheckMenuItem(hMenu, IDM_SLIPSTREAM, MF_UNCHECKED) ;
    CheckMenuItem(hMenu, IDM_NOSLIPSTREAM, MF_CHECKED) ;
    }
  CheckMenuItem(hMenu, IDM_ROYALYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_TOPGALYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_TOPSAILYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_LOWERYARDS, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, iYardLevelShown+310, MF_CHECKED) ;
  CheckMenuItem(hMenu, IDM_YARDANGLE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_EXPOSEDPERCENTAGE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_EFFECTIVEAREA, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_USEFULPERCENTAGE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_USEFULFORCE, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_FORWARDTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_LEEWARDTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_ROTATIONALTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, IDM_HEELINGTHRUST, MF_UNCHECKED) ;
  CheckMenuItem(hMenu, iDisplayParam+321, MF_CHECKED) ;

  // remove bearing from plan view if approprite 
  if (fNow > (fBearingTime + 1))
    {
    hdc=GetDC (hwnd) ;
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xBearing+1, yBearing, xBearing - 25, yBearing + 12) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    ReleaseDC (hwnd, hdc) ;
    }

  // change wind direction if necessary
  if (bWindDirChange)
    {
    fWindGap = fNewWindDirection - fWindDirection ;
    if (fWindGap > 180.)
      fWindGap = fWindGap - 360.0 ;
    if (fWindGap < -180.0)
      fWindGap = fWindGap + 360.0 ;
    if (fabs(fWindGap) > 0.5)
      {
      if (fWindGap > 0.)
        {
        fWindDirection = fWindDirection + fDeltaWindDir ;
        if (fWindDirection > 359.9)
          fWindDirection = fWindDirection - 360.0 ;
        }
      else
        {
        fWindDirection = fWindDirection - fDeltaWindDir ;
        if (fWindDirection < 0.)
          fWindDirection = fWindDirection + 360.0 ;
        }  
      sprintf(sMessage, "%3d", (int) fWindDirection) ; 
      if (strnicmp (sMessage, sOldWindDirection, 3))
        {
        hdc=GetDC (hwnd) ;
        SetTextColor(hdc, RGB(0,100,0)) ; // olive green
        yThis = yWind + iTextHeight ;
        SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
        TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "  %3.0f ", fWindDirection)); 
        SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
        strncpy(sOldWindDirection,sMessage,3) ; 
        bPlanViewIsChanged = TRUE ;
        SetTextColor(hdc, RGB(0,0,0)) ; // black
        ReleaseDC (hwnd, hdc) ;
        }
      }      
    else
      {
      bWindDirChange = FALSE ;
      fWindDirection = fNewWindDirection ;
      sprintf(sMessage, "Wind now steady from %3.0f ", fWindDirection); 
      ShowMessage () ;
      }
    }

  // change wind speed if necessary
  if (bWindSpeedChange)
    {
    if (fNewWindSpeed > fWindSpeed)
      fWindSpeed = fWindSpeed + fDeltaWindSpeed ;
    else
      fWindSpeed = fWindSpeed - fDeltaWindSpeed ;
    sprintf(sMessage, "%3d", (int) fWindSpeed) ; 
    if (strnicmp (sMessage, sOldWindSpeed, 3))
      {
      hdc=GetDC (hwnd) ;
      SetTextColor(hdc, RGB(0,100,0)) ; // olive green
      yThis = yWind + iTextHeight ;
      TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "at %3.0f kt  ", fWindSpeed)); 
      ReleaseDC (hwnd, hdc) ;
      strncpy(sOldWindSpeed,sMessage,3) ; 
      bPlanViewIsChanged = TRUE ;
      SetTextColor(hdc, RGB(0,0,0)) ; // black
      }
    if (fabs(fNewWindSpeed-fWindSpeed)< 0.5) 
      {
      bWindSpeedChange = FALSE ;
      fWindSpeed = fNewWindSpeed ;
      sprintf(sMessage, "Wind now steady at %3.0f kts", fWindSpeed); 
      ShowMessage () ;
      }
    }

  // Automatic Crewmen
  if (bYardsmen)
    ManBraces() ;
  if (bHelmsman && iTackStatus==0)
    ManHelm() ;

  // change helm if necessary
  if (iNewHelmAngle != iHelmAngle)
    {
    iDelta = iNewHelmAngle - iHelmAngle ;
    bPlanViewIsChanged = TRUE ;
    if (abs(iDelta) <= 5)
      iHelmAngle = iHelmAngle + Sgn((float) iDelta) ;
    else
      iHelmAngle = iHelmAngle + Sgn((float) iDelta) * 5 ;
    ShowActualHelmAngle() ;
    } 

  // change yard positions if necessary
  for (iI = 0 ; iI <= 3 ; iI++)
    {
    iDelta = iNewYardAngle[iI] - iYardAngle[iI] ;
    if (iDelta != 0)
      {
      if (abs(iDelta) <= 5)
        iYardAngle[iI] = iYardAngle[iI] + Sgn((float) iDelta) ;
      else
        iYardAngle[iI] = iYardAngle[iI] + Sgn((float) iDelta) * 5 ;
      switch (iI)
        {
        case 0 :
          fSailAngle[0] = iYardAngle[0] ;
          fSailAngle[1] = iYardAngle[0] ;
          fSailAngle[2] = iYardAngle[0] ;
          fSailAngle[3] = iYardAngle[0] ;
          break ;
        case 1 :
          fSailAngle[4] = iYardAngle[1] ;
          fSailAngle[5] = iYardAngle[1] ;
          fSailAngle[6] = iYardAngle[1] ;
          fSailAngle[7] = iYardAngle[1] ;
          break ;
        case 2 :
          fSailAngle[8] = iYardAngle[2] ;
          fSailAngle[9] = iYardAngle[2] ;
          fSailAngle[10] = iYardAngle[2] ;
          break ;
        case 3 :
          fSailAngle[11] = iYardAngle[3] ;
        }
      bPlanViewIsChanged = TRUE ;
      ShowActualYardAngle(iI) ;
      }
    }

  // scroll message display if necessary
  if (iMessageNr > -1)
    {
    if (fNow >= fMessageTime[0]+10.)
      {
      hdc=GetDC (hwnd) ;
      for (iI = 0; iI <= 4; iI++)
        {
        strcpy(sShipMessage[iI],sShipMessage[iI+1]) ;
        fMessageTime[iI] = fMessageTime[iI+1] ;
        SelectObject (hdc,GetStockObject(NULL_PEN)) ;
        Rectangle (hdc, xMessage, yMessage + iI*iTextHeight, xMessage+140, yMessage + (iI+1)*iTextHeight+2) ;
        SelectObject (hdc,GetStockObject(BLACK_PEN)) ;
        TextOut (hdc, xMessage, yMessage + iI*iTextHeight, sShipMessage[iI], strlen(sShipMessage[iI])) ; 
        }
      sShipMessage[5][0]='\0' ;
      fMessageTime[5] = 999999. ;
      SelectObject (hdc,GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xMessage, yMessage+ 5*iTextHeight, xMessage+140, yMessage + 6*iTextHeight+2) ;
      SelectObject (hdc,GetStockObject(BLACK_PEN)) ;
      ReleaseDC (hwnd, hdc) ;
      iMessageNr-- ;
      }
    }

  // adjust sails if necessary
  if (iSailClicks > -1)
    AdjustSails();

  // update position on map
  iMapCounter = iMapCounter + 1 ;
  if (iMapCounter >= 2000)
    {
    bMaxTrack = TRUE ;
    iMapCounter = 1 ;
    }
  fLat[iMapCounter] = fLatitude ;
  fLon[iMapCounter] = fLongtitude ;

  // Sound buzzer if on land
  for (iI = 0 ; iI <= 12 ; iI++)
    {
    fDistance = sqrt((fIsleLat[iI] - fLatitude) * (fIsleLat[iI] - fLatitude) + (fIsleLong[iI] - fLongtitude) * (fIsleLong[iI] - fLongtitude)) ;
    if (fDistance < fIsleRadius[iI])
      MessageBeep(0) ;
    }

  // Show number of update cycles
  hdc=GetDC (hwnd) ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xLeft, yBottomLine, xLeft + 20, yBottomLine + iTextHeight-1) ;
  SetTextColor(hdc, RGB(150,150,150)) ; // dark grey
  TextOut (hdc, xLeft, yBottomLine, sMessage, sprintf (sMessage, "%1d", (2*iCycles))); 
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  ReleaseDC (hwnd, hdc) ;
  iCycles = 0 ;

  }    

/* --------------------------------------
   DerivedSailParameters
   Version date = 99.09.21
----------------------------------------*/

void DerivedSailParameters()
  {
  int    iI ;
  float  fTemp ;
  
  // Calculate heights of head, foot and effective centre of sails
  // above centre of rotation and position of all corners on SailPlan
  fTemp = fLowestSailFoot[0] ;
  for (iI = 3 ; iI >= 0 ; iI--)
    {
    fSailFoot[iI] = fTemp ;
    fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;
    fTemp = fTemp + fSailDepth[iI] + 3.0 ;
    }
  fTemp = fLowestSailFoot[1] ;
  for (iI = 7 ; iI >= 4 ; iI--)
    {
    fSailFoot[iI] = fTemp ;
    fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;
    fTemp = fTemp + fSailDepth[iI] + 3.0 ;
    }
  fTemp = fLowestSailFoot[2] ;
  for (iI = 11 ; iI >= 8 ; iI--)
    {
    fSailFoot[iI] = fTemp ;
    fSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] ;
    fTemp = fTemp + fSailDepth[iI] + 3.0 ;
    }
  for (iI=0 ; iI <= 10 ; iI++)
    {
    fPlanHeadLeft[iI] = -fHeadWidth[iI] / 2.0 ;
    fPlanHeadRight[iI] = fHeadWidth[iI] / 2.0 ;
    fPlanFootLeft[iI] = -fFootWidth[iI] / 2.0 ;
    fPlanFootRight[iI] = fFootWidth[iI] / 2.0 ;
    }
  
  // jib
  switch (iJibSet)
    {
    case 0 :
      SetWindowText(hwndButton[6], "Working") ;
      SetWindowText(hwndButton[7], "Storm") ;
      break ;
    case 1 :
      SetWindowText(hwndButton[6], "Storm") ;
      SetWindowText(hwndButton[7], "Large") ;
      break ;
    case 2 :
      SetWindowText(hwndButton[6], "Large") ;
      SetWindowText(hwndButton[7], "Working") ;
    }
  if (iSailStatus[12] == 4)
    {
    // furled, can be changed
    EnableWindow (hwndJib, FALSE) ;
    EnableWindow (hwndButton[6], TRUE) ;
    EnableWindow (hwndButton[7], TRUE) ;
    }  
  else
    {
    EnableWindow (hwndJib, TRUE) ;
    EnableWindow (hwndButton[6], FALSE) ;
    EnableWindow (hwndButton[7], FALSE) ;
    }

  JibSelect () ;

  // mizzen
  fPlanHeadLeft[11] = -fHeadWidth[11] - 1.0 ;
  fPlanHeadRight[11] = -1.0 ;
  fPlanFootLeft[11] = fPlanHeadLeft[11] ;
  fPlanFootRight[11] = fPlanHeadRight[11] ;

  for (iI = 0 ; iI < 13 ; iI++)
   {
    fEffSailHead[iI] = fSailFoot[iI] + fSailDepth[iI] * fSetFraction[iSailStatus[iI]] ;
    fMeanSailHeight[iI] = (fEffSailHead[iI] + fSailFoot[iI]) / 2.0 ;
   } 

  // Randomized actual maximum winds speeds for sails
  for (iI = 0 ; iI < 13 ; iI++)
    fMaxSpeed[iI] = fTheorMaxSpeed[iI] * (0.9 + 0.2 * (float) rand()/RAND_MAX) ;

  // Mastheights
  fMastHeight[0] = fLowestSailFoot[0] ;
  for (iI =3 ; iI >= 0 ; iI--)
    fMastHeight[0] = fMastHeight[0] + fSailDepth[iI] + 3.0 ;
  fMastHeight[1] = fLowestSailFoot[1] ;
  for (iI = 7 ; iI >= 4 ; iI--) 
    fMastHeight[1] = fMastHeight[1] + fSailDepth[iI] + 3.0 ;
  fMastHeight[2] = fLowestSailFoot[2] ;
  for (iI = 11 ; iI >= 8 ; iI--) 
    fMastHeight[2] = fMastHeight[2] + fSailDepth[iI] + 3.0 ;

  // Scale factor  for SailPlan
  fSailScale = 850.0 / (fMastHeight[1]+10.0) ;
  if ( 1000.0 / (fMastDistance[3] - fMastDistance[2] + fHeadWidth[11] + 20.) < fSailScale )
    fSailScale = 1000.0 / (fMastDistance[3] - fMastDistance[2] + fHeadWidth[11] + 20.) ;
  if ( 1000.0 / (fShipLength + 20.) < fSailScale )
    fSailScale = 1000.0 / (fShipLength + 20.) ;

  CanShadow() ;

  }

/* --------------------------------------
   DerivedShipParameters
   Version date = 02.12.10
----------------------------------------*/

void DerivedShipParameters()
  {
  int   iI ;

  // Dynamic parameters
  fCSABow = fShipWidth * fShipDraught * fBowFactor ; // Cross Sectional Area
  fCSAStern = fShipWidth * fShipDraught * fSternFactor ; 
  fCSASide = fShipLength * fShipDraught * fSideFactor ;
  fShipMass = fShipTons * fMassFactor ;
  fRotationalInertia = fShipMass * fRotatInertFactor * 
    (fShipLength * fShipLength + 3.0 * fShipWidth * fShipWidth / 4.0) / 12.0 ; // cylinder
  fVerticalInertia = fHeelInertFactor * (fShipMass / 2.0) * (fShipWidth / 2.0) * 
     (fShipWidth / 2.0) ;
  fHeelResistFactor = fShipLength * fHeelDragFactor ;
  fMaxHeelMoment = fShipWidth / 4.0 ;

  iYardAngle[0] = fSailAngle[0] ;
  iYardAngle[1] = fSailAngle[4] ;
  iYardAngle[2] = fSailAngle[8] ;
  for (iI=0 ; iI<3 ; iI++)
    {
    iNewYardAngle[iI] = iYardAngle[iI] ;
    ShowActualYardAngle(iI) ;
    SetScrollRange (hwndYard[iI], SB_CTL, -iMaxYardAngle[iI], iMaxYardAngle[iI], FALSE) ;
    SetScrollPos (hwndYard[iI], SB_CTL, iYardAngle[iI], TRUE) ;
    }
  iYardAngle[3] = fSailAngle[11] ;
  iNewYardAngle[3] = iYardAngle[3] ;
  ShowActualYardAngle(3) ;
  SetScrollRange (hwndYard[3], SB_CTL, -iMaxYardAngle[3], iMaxYardAngle[3], FALSE) ;
  SetScrollPos (hwndYard[3], SB_CTL, -iYardAngle[3], TRUE) ;
  iNewHelmAngle = iHelmAngle ;
  ShowActualHelmAngle() ;
  SetScrollRange (hwndHelm, SB_CTL, -iMaxHelmAngle, iMaxHelmAngle, FALSE) ;
  SetScrollPos (hwndHelm, SB_CTL, iHelmAngle, TRUE) ;
  SetScrollRange (hwndJib, SB_CTL, 0, iMaxJibAngle, FALSE) ;
  SetScrollPos (hwndJib, SB_CTL, (int) fSailAngle[12], TRUE) ;

  // map
  fMapRadius = fMapWidth / 2.0 ;
  fOneThird = 0.67 * fMapRadius ;
  bMapMustBePainted = TRUE ;

  // wind
  fRelWindAngle = fHeading - (fWindDirection + 180.0) ;
  if (fRelWindAngle < -180.0) 
    fRelWindAngle = fRelWindAngle + 360.0 ;
  if (fRelWindAngle > 180.0)
    fRelWindAngle = fRelWindAngle - 360.0 ;
  fNewWindDirection = fWindDirection ;
  fNewWindSpeed = fWindSpeed ;

  // hull on plan view
  fScale = max(fMastDistance[3],fShipLength/2)+10 ;
  for (iI = 0 ; iI <= 7 ; iI++)
    { 
    pHull[iI].x = xHullTemplate[iI] * 0.5 * fShipWidth / fScale ;  
    pHull[iI].y = yHullTemplate[iI] * 0.5 * fShipLength / fScale ;  
    }
  fScale = 1000.0 / fScale ;

  //hull on stern view
  pStern[0].x= -0.4 * fShipWidth ;
  pStern[0].y= fQDeckHeight ;
  pStern[1].x= -fShipWidth ;
  pStern[1].y= -fShipDraught*1.6 ;
  pStern[2].x= fShipWidth ;
  pStern[2].y= -fShipDraught*1.6 ;
  pStern[3].x= 0.4 * fShipWidth ;
  pStern[3].y= fQDeckHeight ;

  }

/* --------------------------------------
   HelmAdjust
   Version date = 99.10.03
----------------------------------------*/

void HelmAdjust()
  {
  HDC      hdc ;
  int      yThis ;

  if (bHelmsman)
    SetScrollPos (hwndHelm, SB_CTL, -iNewHelmAngle, TRUE) ;
  else
    {
    hdc = GetDC (hwnd) ;
    iNewHelmAngle = GetScrollPos (hwndHelm, SB_CTL) ;
    yThis = ySliders[4]-5 ;
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xSelected, yThis , xSelected + 50, yThis + iTextHeight );
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    switch (Sgn(iNewHelmAngle))
      {
      case 1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iNewHelmAngle))) ; 
        break ;
      case -1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewHelmAngle))) ; 
        break ;
      default :
        TextOut (hdc, xSelected, yThis, "amidships",9) ; 
      }
    ReleaseDC (hwnd, hdc) ;
    } 
  }

/* --------------------------------------
   Sgn: extracts sign of argument (1=+; 0=0; -1=-)
   Version date = 99.03.14
----------------------------------------*/

int Sgn (float fNumber)
  {
  int    iSign ;
  iSign = 0 ;
  if (fNumber < 0)
    iSign = -1 ;
  if (fNumber > 0)
    iSign = 1 ;
  return iSign;
  }

/* --------------------------------------
   ShowActualHelmAngle
   Version date = 99.10.03
----------------------------------------*/

void ShowActualHelmAngle ()
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[4]-5 ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xActual, yThis , xActual + 50, yThis + iTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  switch (Sgn(iHelmAngle))
    {
    case 1 :
      TextOut (hdc, xActual, yThis, sMessage, 
        sprintf(sMessage, "%3d  stbd", abs(iHelmAngle))) ; 
      break ;
    case -1 :
      TextOut (hdc, xActual, yThis, sMessage, 
        sprintf(sMessage, "%3d  port", abs(iHelmAngle))) ; 
      break ;
    default :
      TextOut (hdc, xActual, yThis, "amidships",9) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   ShowActualJibAngle
   Version date = 99.10.03
----------------------------------------*/
void ShowActualJibAngle ()
  {
  HDC   hdc;

  if (iSailStatus[12] != 4) 
    SetScrollPos (hwndJib, SB_CTL, (int) fabs(fSailAngle[12]), TRUE) ;
  hdc = GetDC (hwnd) ;
  if (iSailStatus[12] == 4)
    SetTextColor(hdc, RGB(120,120,120)) ;  // gray
  TextOut (hdc, xJib, yMessage+20, "Jib sheet:",10) ;
  sprintf (sMessage, "%3d ", (int) fabs(fSailAngle[12])) ; 
  SelectObject (hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xJib+53, yMessage+20, xJib+67, yMessage+35) ;
  SetTextAlign (hdc, TA_RIGHT|TA_TOP) ;
  TextOut (hdc, xJib+66, yMessage+20, sMessage, 5) ; 
  SetTextAlign (hdc, TA_LEFT|TA_TOP) ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  ReleaseDC (hwnd, hdc) ;
  bSailPlanIsChanged = TRUE ;
  }

/* --------------------------------------
   ShowActualYardAngle
   Version date = 02.12.11
----------------------------------------*/

void ShowActualYardAngle (int iI)
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[iI]-5 ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xActual, yThis , xActual + 50, yThis + iTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  if (iI==3)
    // spanker boom
    {
    switch (Sgn(iYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xActual, yThis, "amidships",9) ; 
      }
    }
  else
  // yards
    {
    switch (Sgn(iYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xActual, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xActual, yThis, "amidships",9) ; 
      }
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   ShowActualBoomAngle
   Version date = 02.12.11
----------------------------------------*/

void ShowActualBoomAngle (int iI)
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[iI]-5 ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xActual, yThis , xActual + 50, yThis + iTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  switch (Sgn(iYardAngle[iI]))
    {
    case -1 :
      TextOut (hdc, xActual, yThis, sMessage, 
        sprintf(sMessage, "%3d  stbd", abs(iYardAngle[iI]))) ; 
      break ;
    case 1 :
      TextOut (hdc, xActual, yThis, sMessage, 
        sprintf(sMessage, "%3d  port", abs(iYardAngle[iI]))) ; 
      break ;
    default :
      TextOut (hdc, xActual, yThis, "amidships",9) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   ShowSelectedHelmAngle
   Version date = 99.10.03
----------------------------------------*/

void ShowSelectedHelmAngle ()
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[4]-5 ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xSelected, yThis , xSelected + 50, yThis + iTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  switch (Sgn(iNewHelmAngle))
    {
    case 1 :
      TextOut (hdc, xSelected, yThis, sMessage, 
        sprintf(sMessage, "%3d  stbd", abs(iNewHelmAngle))) ; 
      break ;
    case -1 :
      TextOut (hdc, xSelected, yThis, sMessage, 
        sprintf(sMessage, "%3d  port", abs(iNewHelmAngle))) ; 
      break ;
    default :
      TextOut (hdc, xSelected, yThis, "amidships",9) ; 
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   ShowSelectedYardAngle
   Version date = 02.12.11
----------------------------------------*/

void ShowSelectedYardAngle (int iI)
  {
  HDC     hdc ;
  float   yThis ;
  
  hdc = GetDC (hwnd) ;
  yThis = ySliders[iI]-5 ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xActual, yThis , xActual + 50, yThis + iTextHeight );
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  if (iI==3)
    //spanker boom
    {
    switch (Sgn(iNewYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbt", abs(iNewYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xSelected, yThis, "amidships",9) ; 
      }
    }
  else
    //yards
    {
    switch (Sgn(iNewYardAngle[iI]))
      {
      case -1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
        break ;
      case 1 :
        TextOut (hdc, xSelected, yThis, sMessage, 
          sprintf(sMessage, "%3d  stbd", abs(iNewYardAngle[iI]))) ; 
        break ;
      default :
        TextOut (hdc, xSelected, yThis, "amidships",9) ; 
      }
    }
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   SelectSailParameter
   Version date = 02.12.08
----------------------------------------*/

void SelectSailParameter()
  {
  HDC hdc ;
  int yThis ;
  hdc = GetDC (hwnd) ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xParameter, yParameter, xParameter + 130, yParameter+ iTextHeight+1) ;
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  TextOut (hdc, xParameter, yParameter, sParam[iDisplayParam] , 
    strlen(sParam[iDisplayParam])) ; 
  iLastValueToShow = 11 ;
  if (iDisplayParam > 0)
    iLastValueToShow = 12 ;
  if (iDisplayParam > 1)
    iLastValueToShow = 13 ;
  if (iDisplayParam > 4)
    iLastValueToShow = 16 ;
  if (iDisplayParam == 7)
    iLastValueToShow = 17 ;
  if (iDisplayParam > 4 )
    {
    yThis=yMessage ;
    TextOut (hdc, xTotals, yThis, "Total Thrust",12) ;
    yThis = yThis + iTextHeight ;
    TextOut (hdc, xTotals, yThis, "Total Drag",10) ;
    yThis = yThis + iTextHeight ;
    TextOut (hdc, xTotals, yThis, "Resultant",9) ;
    }
  else
    {
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xTotals, yMessage, xTotals+111, yMessage+42) ;
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    }
  if (iDisplayParam == 7)
    {
    SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,RGB(0,100,0))) ; //olive green
    Rectangle (hdc, xParameter + 186, yParameter + 20 , xParameter + 257, yParameter + 50);
    DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN))) ;
    TextOut (hdc, xParameter+188, yParameter+22, "Effect of Heel",14) ;
    }
  else
    {
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xParameter + 186, yParameter + 20 , xParameter + 258, yParameter + 51 );
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    } 
  if (iDisplayParam == 0)
    Rectangle (hdc, xSailParameter[12]-38, ySailParameter[12]-2, xSailParameter[12]+17, 
      ySailParameter[12]+14) ;
  if (iDisplayParam < 2)
    Rectangle (hdc, xSailParameter[13]-38, ySailParameter[13]-2, xSailParameter[13]+17, 
      ySailParameter[13]+14) ;
  ReleaseDC (hwnd, hdc) ;
  }

/* --------------------------------------
   YardAdjust
   Version date = 02.12.11
----------------------------------------*/

void YardAdjust(int iIndex)
  {
  HDC      hdc ;
  int      i1, i2, iI ;
  int      yThis ;

  hdc = GetDC (hwnd) ;
  i1=iIndex ;
  i2=iIndex ;
  //.....checkboxes
  if ((int) SendMessage(hwndCheck[0],BM_GETCHECK,0,0) && (iIndex == 0 || iIndex == 1))
    {
    i1 = 0 ;
    i2 = 1 ;
    }
  if ((int) SendMessage(hwndCheck[1],BM_GETCHECK,0,0) && (iIndex == 1 || iIndex == 2))
    {
    i1 = 1 ;
    i2 = 2 ;
    }
  if ((int) SendMessage(hwndCheck[0],BM_GETCHECK,0,0) && 
    (int) SendMessage(hwndCheck[1],BM_GETCHECK,0,0) && (iIndex < 3))
    {
    i1 = 0 ;
    i2 = 2 ;
    }
  //adjust yard(s)
  for (iI=i1 ; iI<=i2 ; iI++)
    {
    SetScrollPos (hwndYard[iI], SB_CTL, GetScrollPos (hwndYard[iIndex], SB_CTL), TRUE) ;
    if (iI==3)
      iNewYardAngle[iI] = -GetScrollPos (hwndYard[iIndex], SB_CTL) ;
    else
      iNewYardAngle[iI] = GetScrollPos (hwndYard[iIndex], SB_CTL) ;
    yThis = ySliders[iI]-5 ;
    SelectObject(hdc,GetStockObject(NULL_PEN)) ;
    Rectangle (hdc, xSelected, yThis , xSelected + 50, yThis + iTextHeight );
    SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
    if (iI==3)
      // spanker boom
      {
      switch (Sgn(iNewYardAngle[iI]))
        {
        case -1 :
          TextOut (hdc, xSelected, yThis, sMessage, 
            sprintf(sMessage, "%3d  stbd", abs(iNewYardAngle[iI]))) ; 
          break ;
        case 1 :
          TextOut (hdc, xSelected, yThis, sMessage,
            sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
          break ;
        default :
          TextOut (hdc, xSelected, yThis, "amidships",9) ; 
        }
      }
    else
      // square yards
      {
      switch (Sgn(iNewYardAngle[iI]))
        {
        case -1 :
          TextOut (hdc, xSelected, yThis, sMessage, 
            sprintf(sMessage, "%3d  port", abs(iNewYardAngle[iI]))) ; 
          break ;
        case 1 :
          TextOut (hdc, xSelected, yThis, sMessage,
            sprintf(sMessage, "%3d  stbd", abs(iNewYardAngle[iI]))) ; 
          break ;
        default :
          TextOut (hdc, xSelected, yThis, "amidships",9) ; 
        }
      }
    } 
  ReleaseDC (hwnd, hdc) ;
  }


