/* --------------------------------------
   Set.C Sets Up Display 
   Version date = 02.12.11
----------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include "external.h"

// Prototypes:
void   NewFrame (HDC, int, int, int, int, int, int) ;
void   SetUpDisplay() ;
void   ShowParameters() ;

extern void HelmAdjust () ;
extern void SelectSailParameter() ;
extern void ShowActualHelmAngle () ;
extern void ShowActualJibAngle () ;
extern void ShowActualYardAngle (int) ;
extern void ShowMapWidth () ;
extern void ShowYardShown () ;
extern int  Sgn (float) ;
extern void YardAdjust (int) ;
        
/* --------------------------------------
   NewFrame
   Version date = 99.08.19
----------------------------------------*/

void NewFrame(HDC hdc, int x1, int y1, int iWide, int iHigh, int iStyle, int iFrameWidth)
  {
  // "iStyle" should be 1=inward" or 0=outward"
  // "iFrameWidth" is the size you want the frame to be.

  COLORREF   crLt, crRb;

  // Draw Highlight and Shadow lines
  if (iStyle == 1)
    {
    crLt = RGB(128,128,128) ;
    crRb = RGB(255,255,255) ;
    } 
  else
    {
    crRb = RGB(128,128,128) ;
    crLt = RGB(255,255,255) ;
    }
  SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,crLt)) ;
  MoveToEx (hdc, x1-iFrameWidth, y1-iFrameWidth, NULL);
  LineTo (hdc, x1 + iWide + iFrameWidth, y1 - iFrameWidth) ;  
  MoveToEx (hdc, x1-iFrameWidth, y1-iFrameWidth, NULL);
  LineTo (hdc, x1 - iFrameWidth, y1 + iHigh + iFrameWidth) ;
  DeleteObject(SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,crRb))) ;
  MoveToEx (hdc, x1+iWide+iFrameWidth, y1-iFrameWidth, NULL);
  LineTo (hdc, x1 + iWide + iFrameWidth, y1 + iHigh + iFrameWidth) ;  
  MoveToEx (hdc, x1+iWide+iFrameWidth, y1+iHigh+iFrameWidth, NULL);
  LineTo (hdc, x1 - iFrameWidth, y1 + iHigh + iFrameWidth ) ;
  DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN))) ;
  }

/* --------------------------------------
   SetUpDisplay
   Version date = 02.12.11
----------------------------------------*/

void SetUpDisplay()
  {
  HDC      hdc ;
  int      iI ;
  int      yThis ;

  iTextHeight = 13 ;

  // primary positioning elements
  xLeft = 5 ;
  xWind= 35 ;
  xColon = 65 ;
  xMessage = 90 ;
  xValues = 100 ;
  xAuto = 246 ;
  xBearing= 248 ;
  xParameter = 274 ;
  xTotals = 344 ;
  xJib = 472 ;
  xSliders = 582 ;
  yBearing = 5 ;
  yParameter = 284 ;
  yFirst = 280 ;
  yWind = 380 ;
  yMessage = 430 ;
  yTotals = 440 ;
  ySliders[0] = 302 ;
  yBottomLine = 500 ;

  ySliders[1] = ySliders[0]+35 ;
  ySliders[2] = ySliders[1]+35 ;
  ySliders[3] = ySliders[2]+35 ;
  ySliders[4] = ySliders[3]+45  ;
  xSelected = xSliders + 70 ;
  xActual = xSliders + 130 ;

  xSailParameter[0] = xParameter + 160 ;
  xSailParameter[1] = xParameter + 160 ;
  xSailParameter[2] = xParameter + 160 ;
  xSailParameter[3] = xParameter + 160 ;
  xSailParameter[4] = xParameter + 100 ;
  xSailParameter[5] = xParameter + 100 ;
  xSailParameter[6] = xParameter + 100 ;
  xSailParameter[7] = xParameter + 100 ;
  xSailParameter[8] = xParameter + 40 ;
  xSailParameter[9] = xParameter + 40 ;
  xSailParameter[10] = xParameter + 40 ;
  xSailParameter[11] = xParameter + 40 ;
  xSailParameter[12] = xParameter + 220 ;
  xSailParameter[13] = xParameter + 40 ;
  xSailParameter[14] = xTotals + 95 ;
  xSailParameter[15] = xTotals + 95 ;
  xSailParameter[16] = xTotals + 95 ;
  xSailParameter[17] = xParameter + 235 ;
  ySailParameter[0] = yParameter + 20 ;
  ySailParameter[1] = yParameter + 40 ;
  ySailParameter[2] = yParameter + 60 ;
  ySailParameter[3] = yParameter + 80 ;
  ySailParameter[4] = yParameter + 20 ;
  ySailParameter[5] = yParameter + 40 ;
  ySailParameter[6] = yParameter + 60 ;
  ySailParameter[7] = yParameter + 80 ;
  ySailParameter[8] = yParameter + 20 ;
  ySailParameter[9] = yParameter + 40 ;
  ySailParameter[10] = yParameter + 60 ;
  ySailParameter[11] = yParameter + 80 ;
  ySailParameter[12] = yParameter + 80 ;
  ySailParameter[13] = yParameter + 117 ;
  ySailParameter[14] = yMessage ;
  ySailParameter[15] = yMessage + iTextHeight+1 ;
  ySailParameter[16] = yMessage + 2 * (iTextHeight+1);
  ySailParameter[17] = yParameter + 35 ;

  hdc = GetDC (hwnd) ;
  SetWindowOrgEx (hdc, xScrollPos, yScrollPos, NULL);

  // child windows
  MoveWindow (hwndChild[0],5-xScrollPos,20-yScrollPos,xChildPixels,yChildPixels, TRUE) ;
  bPlanViewIsChanged = TRUE ;
  MoveWindow (hwndChild[1],270-xScrollPos,20-yScrollPos,xChildPixels,yChildPixels, TRUE) ;
  bSailPlanIsChanged = TRUE ;
  MoveWindow (hwndChild[2],535-xScrollPos,20-yScrollPos,iMapPixels,iMapPixels, TRUE) ;
  bMapMustBePainted= TRUE ;
  MoveWindow (hwndChild[3],185-xScrollPos,278-yScrollPos,70,143, TRUE) ;
  bSternViewIsChanged = TRUE ;

  // sliders 
  for (iI=0; iI<4; iI++)
    MoveWindow(hwndYard[iI], xSliders-xScrollPos, ySliders[iI]+10-yScrollPos, 180, 15, TRUE) ;
  MoveWindow(hwndHelm, xSliders-xScrollPos, ySliders[4]+10-yScrollPos, 180, 20, TRUE) ;
  MoveWindow(hwndJib, xJib-xScrollPos, yMessage+35-yScrollPos, 91, 15, TRUE) ;

  // buttons 
  MoveWindow(hwndButton[0], 659-xScrollPos,0-yScrollPos,60,17, TRUE) ; // zoom in
  if (fMapWidth < 0.1)
    EnableWindow (hwndButton[0], FALSE) ;
  else
    EnableWindow (hwndButton[0], TRUE) ;
  MoveWindow(hwndButton[1], 721-xScrollPos,0-yScrollPos,60,17, TRUE) ; // zoom out
  if (fMapWidth > 20.)
    EnableWindow (hwndButton[1], FALSE) ;
  else
    EnableWindow (hwndButton[1], TRUE) ;
  MoveWindow(hwndButton[2], 272-xScrollPos,0-yScrollPos,80,17, TRUE) ; // increase sail
  if (iMadeSail == 8)
    EnableWindow (hwndButton[2], FALSE) ;
  else
    EnableWindow (hwndButton[2], TRUE) ;
  MoveWindow(hwndButton[3], 354-xScrollPos,0-yScrollPos,80,17, TRUE) ; // decrease sail
  if (iMadeSail == 0)
    EnableWindow (hwndButton[3], FALSE) ;
  else
    EnableWindow (hwndButton[3], TRUE) ;
  MoveWindow(hwndButton[4], xAuto-xScrollPos,yMessage+3-yScrollPos,78,17, TRUE) ; // helmsman
  MoveWindow(hwndButton[5], xAuto-xScrollPos,yMessage+40-yScrollPos,78,17, TRUE) ; // yardsmen
  MoveWindow(hwndButton[6], xJib-xScrollPos,yMessage+57-yScrollPos,45,19, TRUE) ; // jib selector #1
  MoveWindow(hwndButton[7], xJib+47-xScrollPos,yMessage+57-yScrollPos,45,19, TRUE) ; // jib selector #2
  MoveWindow(hwndButton[8], xTotals-xScrollPos,yMessage+57-yScrollPos,110,23, TRUE) ; // pause/restart

  // checkboxes 
  MoveWindow(hwndCheck[0], xSliders+25-xScrollPos,ySliders[0]-5-yScrollPos,12,12, TRUE) ;
  MoveWindow(hwndCheck[1], xSliders+38-xScrollPos,ySliders[2]-5-yScrollPos,12,12, TRUE) ;
  MoveWindow(hwndCheck[2], 538-xScrollPos,26+iMapPixels-yScrollPos,15,12, TRUE) ;
  TextOut (hdc, 538+20,26+iMapPixels,"Track ship",10) ; 

  // line for map width
  MoveToEx (hdc, 765, 20, NULL) ;
  LineTo (hdc, 776, 20) ;
  MoveToEx (hdc, 770, 20, NULL) ;
  LineTo (hdc, 770, 20+iMapPixels) ;
  MoveToEx (hdc, 765, 20+iMapPixels, NULL) ;
  LineTo (hdc, 776, 20+iMapPixels) ;
  TextOut (hdc, 760, 115+iTextHeight, "n.m.", 4) ; 

  // legents for basic parameters
  yThis = yFirst ;
  TextOut (hdc, xLeft, yThis, "Latitude",8) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + iTextHeight ;
  TextOut (hdc, xLeft, yThis, "Longtitude",10) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + iTextHeight ;
  TextOut (hdc, xLeft, yThis, "Heading",7) ; 
  TextOut (hdc, xColon, yThis, ":",1) ;
  TextOut (hdc, xValues, yThis, " ",2) ;
  yThis = yThis + iTextHeight ;
  TextOut (hdc, xLeft, yThis, "Speed",5) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  TextOut (hdc, xValues, yThis, " kt",3) ;
  yThis = yThis + iTextHeight ;
  TextOut (hdc, xLeft, yThis, "Leeway",6) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + iTextHeight ;
  TextOut (hdc, xLeft, yThis, "Rot (/min)",11) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + iTextHeight ;
  TextOut (hdc, xLeft, yThis, "Heel angle",10) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + iTextHeight ;

  // force repainting
  strcpy(sOldHeading,"???") ;
  strcpy(sOldSpeed,"???") ;
  strcpy(sOldLeewayAngle,"???") ;
  strcpy(sOldDegreesPerMin,"???") ;
  strcpy(sOldHeelingAngle,"???") ;

   // legends for wind parameters
  yThis= yWind ; 
  SetTextColor(hdc, RGB(0,100,0)) ; // olive green
  TextOut (hdc, xLeft, yThis, "Wind", 4) ; 
  yThis = yThis + iTextHeight ;
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "%3.0f ", fWindDirection)); 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, " at %3.0f kt", fWindSpeed)); 
  yThis = yThis + iTextHeight ;
  SetTextColor(hdc, RGB(255,0,0)) ;  // red
  TextOut (hdc, xLeft, yThis, "Apparent wind", 13) ; 
  yThis = yThis + iTextHeight * 2 ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  TextOut (hdc, xLeft, yThis, "Relative wind", 13) ; 

   // Set up display of sail parameters and totals
  NewFrame(hdc, xParameter-2, yParameter-2, 293, 135, 1, 2) ;
  for (iI = 0 ; iI <= 11 ; iI++)
    {
    MoveToEx (hdc, xSailParameter[iI]-10, ySailParameter[iI]-6, NULL );  
    LineTo (hdc, xSailParameter[iI]-10, ySailParameter[iI]-2) ;  
    }
  for (iI = 3 ; iI <= 11 ; iI=iI+4)
    {
    MoveToEx (hdc, xSailParameter[iI]-10, ySailParameter[iI]+14, NULL );  
    LineTo (hdc, xSailParameter[iI]-10, ySailParameter[iI]+20) ;  
    }
  NewFrame(hdc, xTotals-2, yMessage-2, 112, 50, 1, 2) ;
  MoveToEx (hdc, xParameter+2, yParameter+100, NULL) ;
  LineTo (hdc, xSailParameter[12], yParameter+100) ;  
  LineTo (hdc, xSailParameter[12]-20, yParameter+130) ;  
  LineTo (hdc, xParameter+2, yParameter+130) ;  
  LineTo (hdc, xParameter+2, yParameter+100) ;  

  // set up display of yard and helm sliders
  NewFrame(hdc, xSliders-2, yParameter-2, 188, 215, 1, 2) ;
  yThis=yParameter ;
  TextOut (hdc, xSelected, yThis, "Selected",8) ; 
  TextOut (hdc, xActual, yThis, "Actual",6) ; 
  if (bYardsmen)
    SetTextColor(hdc, RGB(160,160,160)) ; // gray
  yThis=ySliders[0]-5 ;
  TextOut (hdc, xSliders, yThis, "Fore", 4) ; 
  yThis=ySliders[1]-5 ;
  TextOut (hdc, xSliders, yThis, "Main", 4) ; 
  yThis=ySliders[2]-5 ;
  TextOut (hdc, xSliders, yThis, "Mizzen", 6) ; 
  yThis=ySliders[3]-5 ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  TextOut (hdc, xSliders, yThis, "Mizzen boom", 11) ; 
  yThis=ySliders[4]-5 ;
  if (bHelmsman)
    SetTextColor(hdc, RGB(160,160,160)) ;  // gray
  TextOut (hdc, xSliders, yThis, "Helm", 4) ; 
  SetTextColor(hdc, RGB(0,0,0)) ;  // black

  // set up jib display 
  NewFrame(hdc, xJib-2, yMessage-2, 95, 80, 1, 2) ;
  TextOut (hdc, xJib, yMessage+20, "Jib sheet:",10) ;
  SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
  TextOut (hdc, xJib+44, yMessage, sJibName[iJibSet],strlen(sJibName[iJibSet])) ; 

  // set up display of automatic helmsman and yardsmen
  NewFrame(hdc, xAuto-2, yMessage-2, 82, 80, 1, 2) ;
  SelectObject (hdc, GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xAuto, yMessage+21, xAuto+80, yMessage+22+iTextHeight) ;
  SelectObject (hdc, GetStockObject(BLACK_PEN)) ;
  SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
  TextOut (hdc, xAuto+40, yMessage+21, sHelmInstruction,14) ; 
  //SetTextAlign (hdc,TA_LEFT|TA_TOP) ;
  if (bYardsmen)
    TextOut (hdc, xAuto+40, yMessage+58, " man braces ",12) ; 
  else
    TextOut (hdc, xAuto+40, yMessage+58, "(await orders)",14) ; 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;

  // message box
  NewFrame(hdc, xMessage-2, yMessage-2, 140, 80, 1, 2) ;
  if (iMessageNr > -1)
    {
    for (iI = 0 ; iI <= iMessageNr ; iI++)
      { 
      TextOut (hdc, xMessage, yMessage+ iI*iTextHeight, sShipMessage[iI], strlen(sShipMessage[iI])) ; 
      } 
    } 
  ReleaseDC (hwnd, hdc) ;

  // display current values
  ShowYardShown() ;
  ShowMapWidth() ;
  YardAdjust(0) ;
  YardAdjust(1) ;
  YardAdjust(2) ;
  YardAdjust(3) ;
  HelmAdjust() ;
  ShowActualYardAngle(0) ;
  ShowActualYardAngle(1) ;
  ShowActualYardAngle(2) ;
  ShowActualYardAngle(4) ;
  ShowActualJibAngle() ;
  ShowActualHelmAngle() ;
  SelectSailParameter() ;

  }

/* --------------------------------------
   ShowParameters
   Version date = 02.12.11
----------------------------------------*/
void ShowParameters()
  {
  int   iI, iAngle ;
  HDC   hdc ;
  char  cSuffix[5] ;
  int   xThis, yThis ;

  hdc = GetDC (hwnd) ;
  SetWindowOrgEx (hdc, xScrollPos, yScrollPos, NULL);

  SelectObject (hdc, GetStockObject(NULL_PEN)) ;
  SetTextAlign (hdc, TA_RIGHT | TA_TOP) ;

  // Display the basic parameters
  xThis = xValues ;

  // position
  yThis = yFirst ;
  Rectangle (hdc, xThis, yThis, xThis+60, yThis + iTextHeight-1) ;
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  TextOut (hdc, xThis, yThis, sMessage, sprintf(sMessage, "%5.3f", fabs(fLatitude))) ; 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  if (fLatitude >= 0.)
    TextOut (hdc, xThis, yThis, " nm N", 5) ; 
  else
    TextOut (hdc, xThis, yThis, " nm S", 5) ; 
  yThis = yFirst + iTextHeight ;
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  Rectangle (hdc, xThis, yThis, xThis+60, yThis + iTextHeight-1) ;
  TextOut (hdc, xThis, yThis, sMessage, sprintf(sMessage, "%5.3f", fabs(fLongtitude))) ; 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  if (fLongtitude >= 0.)
    TextOut (hdc, xThis, yThis, " nm E", 5) ; 
  else
    TextOut (hdc, xThis, yThis, " nm W", 5) ; 

  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  // heading
  yThis = yThis + iTextHeight ;
  sprintf (sMessage, "%3.0f", fHeading) ; 
  if (strcmp(sMessage , sOldHeading) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis, yThis + iTextHeight-1) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    strcpy(sOldHeading, sMessage) ;
    }

  // speed
  yThis = yThis + iTextHeight ;
  sprintf (sMessage, "%5.1f", fSpeed) ; 
  if (strcmp(sMessage , sOldSpeed) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis, yThis + iTextHeight-1) ;
    if (fSpeed < 0.0)
    SetTextColor(hdc, RGB(255,0,0)) ; // red
    TextOut (hdc, xThis, yThis, sMessage, 5) ; 
    SetTextColor(hdc, RGB(0,0,0)) ; // black
    strcpy(sOldSpeed, sMessage) ;
    }

  // leeway angle
  yThis = yThis + iTextHeight ;
  sprintf (sMessage, "%3.0f", fabs(fLeewayAngle)) ; 
  if (strcmp(sMessage , sOldLeewayAngle) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis + 10 , yThis + iTextHeight-1) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    if (Sgn(fLeewayAngle) > 0)
      TextOut (hdc, xThis, yThis, "  s ", 5) ; 
    else
      if (Sgn(fLeewayAngle) == 0)
        TextOut (hdc, xThis, yThis, "    ", 5) ; 
      else
        TextOut (hdc, xThis, yThis, "  p ", 5) ; 
    strcpy (sOldLeewayAngle, sMessage) ;
    } 

  // rotation
  yThis = yThis + iTextHeight ;
  sprintf (sMessage, "%3.0f", fabs(fDegreesPerMin)) ; 
  if (strcmp(sMessage , sOldDegreesPerMin) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis + 10, yThis + iTextHeight-1) ;
    SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    if (Sgn(fDegreesPerMin) > 0)
      TextOut (hdc, xThis, yThis, "  s ", 5) ; 
    else
      if (Sgn(fDegreesPerMin) == 0)
        TextOut (hdc, xThis, yThis, "    ", 5) ; 
      else
        TextOut (hdc, xThis, yThis, "  p ", 5) ; 
    strcpy (sOldDegreesPerMin, sMessage) ;
    }

  // heel angle
  yThis = yThis + iTextHeight ;
  sprintf (sMessage, "%3.0f", fabs(fHeelingAngle)) ; 
  if (strcmp(sMessage , sOldHeelingAngle) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis + 10, yThis + iTextHeight-1) ;
    SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    if (Sgn(fHeelingAngle) > 0)
      TextOut (hdc, xThis, yThis, "  s ", 5) ; 
    else
      if (Sgn(fHeelingAngle) == 0)
        TextOut (hdc, xThis, yThis, "    ", 5) ; 
      else
        TextOut (hdc, xThis, yThis, "  p ", 5) ;
    strcpy (sOldHeelingAngle, sMessage) ;
    }

  // wind parameters
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  yThis = yWind + 3 * iTextHeight ;
  Rectangle (hdc, xWind - 30, yThis, xWind , yThis + iTextHeight-1) ;
  SetTextColor(hdc, RGB(255,0,0)) ; // red
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "%3.0f  ", fAppWindDirection)); 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "at %3.0f kt  ", fAppWindSpeed)); 
  yThis = yThis + 2 * iTextHeight ;
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  Rectangle (hdc, xWind-30, yThis, xWind+45, yThis + iTextHeight-1) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "%3.0f  ", 180-fabs(fAppWindAngle))); 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  if (fAppWindAngle > 0.98)
    TextOut (hdc, xWind, yThis, "starboard",9); 
  else
    if (fAppWindAngle < -0.98)
      TextOut (hdc, xWind, yThis, "port",4); 
    else
      TextOut (hdc, xWind, yThis, "astern",6); 

  // sail parameters
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  for (iI=0 ; iI <= iLastValueToShow ; iI++)
    {
    switch (iDisplayParam)
      {
      case 0 :
        sprintf (sMessage, "%7.0f", fabs(fSailAngle[iI])) ; 
        if (iI==11)
        // spanker
        {
          switch (Sgn(fSailAngle[iI]))
            {
            case 1 :
              strcpy (cSuffix,  " p") ;
              break ;
            case 0 :
              strcpy (cSuffix,  "  ") ;
              break ;
            case -1 :
              strcpy (cSuffix,  " s") ;
            }
          }
        else
        // other sails
        {
          switch (Sgn(fSailAngle[iI]))
            {
            case 1 :
              strcpy (cSuffix,  " s") ;
              break ;
            case 0 :
              strcpy (cSuffix,  "  ") ;
              break ;
            case -1 :
              strcpy (cSuffix,  " p") ;
            }
          }
        break ;
      case 1 :
        sprintf (sMessage, "%7.0f", fabs(fExposedFract[iI] * 100.0)) ; 
        strcpy (cSuffix,  " % ") ;
        break ;
      case 2 :
        sprintf (sMessage, "%7.0f", fabs(fPerpSailArea[iI])) ; 
        strcpy (cSuffix,  "   ") ;
        break ;
      case 3 :
        sprintf (sMessage, "%7.0f", fabs(fPerpFract[iI] * 100.0)) ; 
        strcpy (cSuffix,  " % ") ;
        break ;
      case 4 :
        sprintf (sMessage, "%7.0f", fabs(fPerpForce[iI])) ; 
        strcpy (cSuffix,  "   ") ;
        break ;
      case 5 :
        sprintf (sMessage, "%7.0f", fabs(fHeadwayForce[iI])) ; 
        switch (Sgn(fHeadwayForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " f ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " a ") ;
           }
         break ;
      case 6 :
        sprintf (sMessage, "%7.0f", fabs(fLeewayForce[iI])) ; 
        switch (Sgn(fLeewayForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " s ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " p ") ;
           }
         break ;
      case 7 :
        sprintf (sMessage, "%7.0f", fabs(fRotatForce[iI])) ; 
        switch (Sgn(fRotatForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " s ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " p ") ;
           }
         break ;
      case 8 :
        sprintf (sMessage, "%7.0f", fabs(fHeelForce[iI])) ; 
        switch (Sgn(fHeelForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " s ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " p ") ;
           }
         break ;
      }
    if (strcmp(sMessage , &sLastValue[iI]) != 0)
      {
      xThis = xSailParameter[iI] ;
      yThis = ySailParameter[iI] ;
      switch (iSailWarning[iI])
        {
        case 0 :
          SetTextColor(hdc, RGB(0,0,0)) ; // black
          break ;
        case 1 :
          SetTextColor(hdc, RGB(127,127,127)) ; // grey
          break ;
        case 2 :
          SetTextColor(hdc, RGB(163,0,0)) ; // dark red
          break ;
        case 3 :
          SetTextColor(hdc, RGB(255,0,0)) ; // red
          break ;
        case 4 :
          SetTextColor(hdc, RGB(255,255,255)) ; // white
          break ;
        }
      if (iI==14)
        SelectObject(hdc,GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xThis-38, yThis-2, xThis+17, yThis+14) ;
      SetTextAlign (hdc, TA_RIGHT | TA_TOP) ;
      TextOut (hdc, xThis, yThis, sMessage, 7); 
      SetTextAlign (hdc, TA_LEFT | TA_TOP) ;
      TextOut (hdc, xThis, yThis, cSuffix, 3); 
      strcpy (&sLastValue[iI], sMessage) ; 
      }
    }

  // alerts
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xWind, yBottomLine, xWind+45, yBottomLine + iTextHeight) ;
  sMessage[0]= '\0' ;
  iI = 0 ;
  if (bAlertSpeed)
    {
    sMessage[0]='S' ; 
    iI = 1 ;
    }
  if (bAlertLeeway)
    {
    sMessage[iI] = 'L' ; 
    iI++ ;
    }
  if (bAlertHeel)
    {
    sMessage[iI] = 'H' ; 
    iI++ ;
    }
  if (bAlertRotat)
    {
    sMessage[iI] = 'R' ; 
    iI++ ;
    }
  if (iI>0)
    {
    SetTextColor(hdc, RGB(255,0,0)) ; // red
    TextOut (hdc, xWind, yBottomLine, sMessage, iI); 
    SetTextColor(hdc, RGB(0,0,0)) ; // black
    }

  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  ReleaseDC (hwnd, hdc) ;
  }
