/* --------------------------------------
   Set.C Sets Up Display 
   Version date = 03.09.17
---------------------------------------*/

/*----------------------------------------------------------------------
COPYRIGHT 2003, 2005 PETER DAVIS
This file is part of "HMS Surprise - Frigate and brig Simulator for
Windows". That program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of the
License, or (at your option) any later version. The file is distributed
in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
PURPOSE.  See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this file; if not, write to the Free Software Foundation,
Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
----------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include "external.h"

// Prototypes:
void   NewFrame (HDC, int, int, int, int) ;
void   SetUpDisplay() ;
void   ShowParameters() ;

extern int  Beaufort (float) ;
extern void SelectSailParameter() ;
extern int  Sgn (float) ;
extern void ShowActualHelmAngle () ;
extern void ShowActualJibAngle () ;
extern void ShowActualStaysailsAngle () ;
extern void ShowActualYardAngle (int) ;
extern void ShowSelectedHelmAngle () ;
extern void ShowSelectedYardAngle (int) ;
extern void ShowMapWidth () ;
extern void ShowYardShown () ;
        
/* --------------------------------------
   NewFrame
   Version date = 03.03.22
----------------------------------------*/

void NewFrame(HDC hdc, int x1, int y1, int iWide, int iHigh)
  {
  SelectObject (hdc,hDarkGrayPen) ;
  MoveToEx (hdc, x1, y1 + iHigh, NULL);
  LineTo (hdc, x1, y1) ;
  LineTo (hdc, x1 + iWide, y1) ;
  SelectObject (hdc,GetStockObject(WHITE_PEN)) ;
  LineTo (hdc, x1 + iWide, y1 + iHigh) ;  
  LineTo (hdc, x1 , y1 + iHigh) ;
  SelectObject (hdc,GetStockObject(BLACK_PEN)) ;
  }

/* --------------------------------------
   SetUpDisplay
   Version date = 03.09.07
----------------------------------------*/

void SetUpDisplay()
  {
  HDC      hdc ;
  int      iI, yChildTop, cyMessage ;
  int      ySailParameterStep, xSternView, xZoomIn, xZoomOut, xIncrease, xDecrease ;
  int      iJibWidth, iJibHeight, cySliders, cxSliders ;
  int      xForeSails, xMainSails, xMizzenSails, xStern, xJibSails, iChildSpacer ;
  int      iButtonHeight, yThis, iTotalShift ;

  iSpacer = 2 ;
  iChildSpacer = 15 ;
  cyTextHeight = 13 ;
  yChildTop = 20;

  xLeft = 5 ;
  xColon =  65 ;
  xSternView = 140 ;
  yFirst = 275 ;
  xWind = 33 ;
  yWind = 393 ;
  xMessage = 84 ;
  yMessage = 425 ;
  xAuto = 258 ;
  xParameter = 240 ;
  xZoomIn = 659 ;
  xZoomOut = 721 ;
  xIncrease = 272 ;
  xDecrease = 352 ;

  xValues = 100 ;
  xBearing= 248 ;
  xTotals = 354 ;
  xJib = 472 ;
  xSliders = 582 ;
  yBearing = 5 ;
  yParameter = 284 ;
  yTotals = 440 ;
  yBottomLine = 500 ;

  cxMessage = 169 ;
  cyMessage = 84 ;
  cxAuto = 86 ;
  iButtonHeight =17 ;
  iParameterWidth = 330 ;
  iParameterHeight = 140 ;
  iTotalsWidth = 114 ;
  iTotalsHeight = 50 ;
  iTotalShift = 95;  
  cyScroll = 15 ;
  cxSliders = 188 ;
  iJibWidth = 95 ;
  iJibHeight = 95 ;
  iOfsetLeft = 38 ;
  iOfsetTop = 2 ;
  iOfsetRight = 17 ;
  iOfsetBottom = 14 ;
  ySailParameterStep = 20 ;
  xStern = 50 ;
  iHeelParamWidth = 74 ;
  iHeelParamHeight = 30 ;

  if (bBrig)
    cySliders = 189 ;
  else
    cySliders = 232 ;

  // primary positioning elements

  ySliders[0] = yFirst+1.5*cyScroll+10 ;
  ySliders[1] = ySliders[0]+2.5*cyScroll ;
  if (bBrig)
    {
    ySliders[3] = ySliders[1]+2.5*cyScroll ;
    ySliders[4] = ySliders[3]+2.5*cyScroll ;
    }
  else
    {
    ySliders[2] = ySliders[1]+2.5*cyScroll ;
    ySliders[3] = ySliders[2]+2.5*cyScroll ;
    ySliders[4] = ySliders[3]+2.5*cyScroll ;
    }

  xSelected = xSliders + 50 ;
  xActual = xSliders + 110 ;

  if (bBrig)
    { 
    xForeSails = 230 ;
    xMainSails = 110 ;
    xJibSails = 290 ;
    xSailParameter[0] = xParameter + xForeSails ; // fore royal
    xSailParameter[1] = xParameter + xForeSails ; // fore topgallant
    xSailParameter[2] = xParameter + xForeSails ; // fore upper topsail
    xSailParameter[3] = xParameter + xForeSails ; // fore lower topsail
    xSailParameter[4] = xParameter + xForeSails ; // fore main
    xSailParameter[5] = xParameter + xMainSails ; // main royal
    xSailParameter[6] = xParameter + xMainSails ; // main topgallant
    xSailParameter[7] = xParameter + xMainSails ; // main upper topsail
    xSailParameter[8] = xParameter + xMainSails ; // main lower topsail
    xSailParameter[9] = xParameter + xMainSails ; // main
    xSailParameter[10] = xParameter + xStern ; // spanker
    xSailParameter[11] = xParameter + xJibSails+6*iSpacer ;  // flying jib
    xSailParameter[12] = xParameter + xJibSails+4*iSpacer ; // outer jib
    xSailParameter[13] = xParameter + xJibSails+2*iSpacer ; // inner jib
    xSailParameter[14] = xParameter + xJibSails ; // fore topmast staysail
    xSailParameter[15] = xParameter + 170 ; // main royal staysail
    xSailParameter[16] = xParameter + 170 ; // main topgallant staysail
    xSailParameter[17] = xParameter + 170 ; // main topmast staysail
    xSailParameter[18] = xParameter + xStern ; // rudder
    xSailParameter[19] = xParameter + 170 ; // hull and superstructure
    xSailParameter[20] = xTotals + iTotalShift ; 
    xSailParameter[21] = xTotals + iTotalShift ;
    xSailParameter[22] = xTotals + iTotalShift ;
    xSailParameter[23] = xParameter+xJibSails+6*iSpacer ; // heel
    ySailParameter[0] = yFirst + cyTextHeight+3*iSpacer ;
    ySailParameter[1] = ySailParameter[0]+ySailParameterStep ;
    ySailParameter[2] = ySailParameter[1]+ySailParameterStep ;
    ySailParameter[3] = ySailParameter[2]+ySailParameterStep ;
    ySailParameter[4] = ySailParameter[3]+ySailParameterStep ;
    ySailParameter[5] = ySailParameter[0] ;
    ySailParameter[6] = ySailParameter[5]+ySailParameterStep ;
    ySailParameter[7] = ySailParameter[6]+ySailParameterStep ;
    ySailParameter[8] = ySailParameter[7]+ySailParameterStep ;
    ySailParameter[9] = ySailParameter[8]+ySailParameterStep ;
    ySailParameter[10] = ySailParameter[9] ;
    ySailParameter[11] = ySailParameter[1] ;
    ySailParameter[12] = ySailParameter[2] ;
    ySailParameter[13] = ySailParameter[3] ;
    ySailParameter[14] = ySailParameter[4] ;
    ySailParameter[15] = yFirst + 40 ;
    ySailParameter[16] = yFirst + 60 ;
    ySailParameter[17] = yFirst + 80 ;
    ySailParameter[18] = ySailParameter[9]+ySailParameterStep ;
    ySailParameter[19] = ySailParameter[18] ;
    ySailParameter[20] = yMessage + 1 ;
    ySailParameter[21] = yMessage + 1 + iSpacer + cyTextHeight ;
    ySailParameter[22] = yMessage + 1 + 2*iSpacer + 2*cyTextHeight ;
    ySailParameter[23] = ySailParameter[0] ;
    }
  else
    { 
    xForeSails = 230 ;
    xMainSails = 170 ;
    xMizzenSails = 110 ;
    xJibSails = 290 ;
    xSailParameter[0] = xParameter + xForeSails ;
    xSailParameter[1] = xParameter + xForeSails ;
    xSailParameter[2] = xParameter + xForeSails ;
    xSailParameter[3] = xParameter + xForeSails ;
    xSailParameter[4] = xParameter + xMainSails ;
    xSailParameter[5] = xParameter + xMainSails ;
    xSailParameter[6] = xParameter + xMainSails ;
    xSailParameter[7] = xParameter + xMainSails ;
    xSailParameter[8] = xParameter + xMizzenSails ;
    xSailParameter[9] = xParameter + xMizzenSails ;
    xSailParameter[10] = xParameter + xMizzenSails ;
    xSailParameter[11] = xParameter + xStern ; // spanker
    xSailParameter[12] = xParameter + xJibSails ; // jib
    xSailParameter[13] = xParameter + xStern ; // rudder
    xSailParameter[14] = xParameter + xMainSails ; // hull and superstructure
    xSailParameter[15] = xTotals + iTotalShift ; 
    xSailParameter[16] = xTotals + iTotalShift ;
    xSailParameter[17] = xTotals + iTotalShift ;
    xSailParameter[18] = xParameter+xJibSails+6*iSpacer ; // heel
    ySailParameter[0] = yFirst + cyTextHeight+3*iSpacer+ySailParameterStep ;
    ySailParameter[1] = ySailParameter[0]+ySailParameterStep ;
    ySailParameter[2] = ySailParameter[1]+ySailParameterStep ;
    ySailParameter[3] = ySailParameter[2]+ySailParameterStep ;
    ySailParameter[4] = ySailParameter[0] ;
    ySailParameter[5] = ySailParameter[1] ;
    ySailParameter[6] = ySailParameter[2] ;
    ySailParameter[7] = ySailParameter[3] ;
    ySailParameter[8] = ySailParameter[0] ;
    ySailParameter[9] = ySailParameter[1] ;
    ySailParameter[10] = ySailParameter[2] ;
    ySailParameter[11] = ySailParameter[3] ;
    ySailParameter[12] = ySailParameter[3] ;
    ySailParameter[13] = ySailParameter[11]+ySailParameterStep ;
    ySailParameter[14] = ySailParameter[13] ;
    ySailParameter[15] = yMessage + 1 ;
    ySailParameter[16] = yMessage + 1 + iSpacer + cyTextHeight ;
    ySailParameter[17] = yMessage + 1 + 2*iSpacer + 2*cyTextHeight ;
    ySailParameter[18] = ySailParameter[0]-ySailParameterStep ;
    }

  hdc = GetDC (hwnd) ;

  SelectObject (hdc, GetStockObject (NULL_PEN)) ;
  SelectObject (hdc, GetStockObject (LTGRAY_BRUSH)) ;
  Rectangle (hdc, 0, 0, 790, 590) ;

  switch (iScreen)
    {
    case 1:  // 800*600 pixels
      cxMap = 220 ;
      cxSternView = 70 ;
      cySternView = 143 ;
      MoveWindow (hwndChild[0], 5-xScrollPos, 20-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ; // plan view
      MoveWindow (hwndChild[1], 270-xScrollPos, 20-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ;  // sail plan
      MoveWindow (hwndChild[2], 535-xScrollPos, 20-yScrollPos, cxMap, cxMap, TRUE) ; // map
      MoveWindow (hwndChild[3], 140-xScrollPos, 275-yScrollPos, cxSternView, cySternView, TRUE) ;  // stern view
      MoveWindow (hwndText, 120-xScrollPos, 2-yScrollPos, 30 , 18, TRUE) ; // planview bearing
      MoveWindow(hwndSlider[0], 582-xScrollPos, 324-yScrollPos, 184, 15, TRUE) ; // fore yards
      MoveWindow(hwndSlider[1], 582-xScrollPos, 361-yScrollPos, 184, 15, TRUE) ; // main yards
      if (bBrig)
        {
        MoveWindow(hwndSlider[2], 0, 0, 0, 0, TRUE) ;                              // mizzen yards
        MoveWindow(hwndSlider[3], 582-xScrollPos, 398-yScrollPos, 184, 15, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 582-xScrollPos, 435-yScrollPos, 184, 15, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 472-xScrollPos, 475-yScrollPos, 95, 15, TRUE) ;  // mainmast staysails
        MoveWindow(hwndButton[7], 0, 0, 0, 0, TRUE) ;                              // smaller jib
        MoveWindow(hwndButton[8], 0, 0, 0, 0, TRUE) ;                              // larger jib
        MoveWindow(hwndCheck[1], 0, 0, 0, 0, TRUE) ;                               // couple mizzen
        }
      else
        {
        MoveWindow(hwndSlider[2], 582-xScrollPos, 398-yScrollPos, 184, 15, TRUE) ;  // mizzen yards
        MoveWindow(hwndSlider[3], 582-xScrollPos, 435-yScrollPos, 184, 15, TRUE) ;  // spanker boom
        MoveWindow(hwndSlider[4], 582-xScrollPos, 472-yScrollPos, 184, 15, TRUE) ;  // helm
        MoveWindow(hwndSlider[6], 0, 0, 0, 0, TRUE) ;                               // mainmast staysails
        MoveWindow(hwndButton[7], 485-xScrollPos, 467-yScrollPos, 70, 15, TRUE) ;   // smaller jib
        MoveWindow(hwndButton[8], 485-xScrollPos, 487-yScrollPos, 70, 15, TRUE) ;   // larger jib
        MoveWindow(hwndCheck[1], 618-xScrollPos, 381-yScrollPos,12,12, TRUE) ;      // couple mizzen
        }
      MoveWindow(hwndSlider[5], 472-xScrollPos, 442-yScrollPos, 95, 15, TRUE) ;  // helmsman
      MoveWindow(hwndCheck[0], 607-xScrollPos, 307-yScrollPos, 12, 12, TRUE) ;   // couple foremast
      MoveWindow(hwndCheck[2], 538-xScrollPos, 246-yScrollPos, 12, 12, TRUE) ;   // track ship
      MoveWindow(hwndButton[0], 659-xScrollPos, 2-yScrollPos, 60, 17, TRUE) ;    // zoom in
      MoveWindow(hwndButton[1], 721-xScrollPos, 2-yScrollPos, 60, 17, TRUE) ;    // zoom out
      MoveWindow(hwndButton[2], 272-xScrollPos, 2-yScrollPos, 80, 17, TRUE) ;    // increase sail
      MoveWindow(hwndButton[3], 352-xScrollPos, 2-yScrollPos, 80, 17, TRUE) ;    // decrease sail
      MoveWindow(hwndButton[4], 260-xScrollPos, 435-yScrollPos, 82, 17, TRUE) ;  // helmsman
      MoveWindow(hwndButton[5], 260-xScrollPos, 473-yScrollPos, 82, 17, TRUE) ;  // yardsmen
      MoveWindow(hwndButton[6], 354-xScrollPos, 485-yScrollPos,110,23, TRUE) ;   // pause/restart
      break ;
    case 2: // 1024*768 pixels
      cxMap = 282 ;
      cxSternView = 90 ;
      cySternView = 183 ;
      MoveWindow (hwndChild[0], 6-xScrollPos, 26-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ; // plan view
      MoveWindow (hwndChild[1], 345-xScrollPos, 26-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ;  // sail plan
      MoveWindow (hwndChild[2], 684-xScrollPos, 26-yScrollPos, cxMap, cxMap, TRUE) ; // map
      MoveWindow (hwndChild[3], 179-xScrollPos, 352-yScrollPos, cxSternView, cySternView, TRUE) ;  // stern view
      MoveWindow (hwndText, 154-xScrollPos, 4-yScrollPos, 30 , 20, TRUE) ;         // planview bearing
      MoveWindow(hwndSlider[0], 745-xScrollPos, 415-yScrollPos, 235, 19, TRUE) ;   // fore mast
      MoveWindow(hwndSlider[1], 745-xScrollPos, 462-yScrollPos, 235, 19, TRUE) ;   // main mast
      if (bBrig)
        {
        MoveWindow(hwndSlider[2], 0, 0, 0, 0, TRUE) ; // mizzen yards
        MoveWindow(hwndSlider[3], 745-xScrollPos, 509-yScrollPos, 235, 19, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 745-xScrollPos, 557-yScrollPos, 235, 19, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 604-xScrollPos, 608-yScrollPos, 121, 19, TRUE) ; // mainmast staysails
        MoveWindow(hwndButton[7], 0, 0, 0, 0, TRUE) ;                              // smaller jib
        MoveWindow(hwndButton[8], 0, 0, 0, 0, TRUE) ;                              // larger jib
        MoveWindow(hwndCheck[1], 0, 0, 0, 0, TRUE) ;                               // couple mizzen
        }
      else
        {
        MoveWindow(hwndSlider[2], 745-xScrollPos, 509-yScrollPos, 235, 19, TRUE) ; // mizzen mast
        MoveWindow(hwndSlider[3], 745-xScrollPos, 557-yScrollPos, 235, 19, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 745-xScrollPos, 605-yScrollPos, 235, 19, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 0, 0, 0, 0, TRUE) ;                              // mainmast staysails
        MoveWindow(hwndButton[7], 623-xScrollPos, 597-yScrollPos, 87, 19, TRUE) ;  // smaller jib
        MoveWindow(hwndButton[8], 623-xScrollPos, 620-yScrollPos, 87, 19, TRUE) ;  // larger jib
        MoveWindow(hwndCheck[1], 790-xScrollPos, 489-yScrollPos,12,12, TRUE) ;     // couple mizzen
        }
      MoveWindow(hwndSlider[5], 604-xScrollPos, 566-yScrollPos, 121, 19, TRUE) ;  // foremast staysails
      MoveWindow(hwndCheck[0], 777-xScrollPos, 393-yScrollPos, 15, 15, TRUE) ;    // couple foremast
      MoveWindow(hwndCheck[2], 688-xScrollPos, 315-yScrollPos, 15, 15, TRUE) ;    // track ship
      MoveWindow(hwndButton[0], 843-xScrollPos, 2-yScrollPos, 77, 22, TRUE) ;     // zoom in
      MoveWindow(hwndButton[1], 922-xScrollPos, 2-yScrollPos, 76, 22, TRUE) ;     // zoom out
      MoveWindow(hwndButton[2], 348-xScrollPos, 2-yScrollPos, 102, 22, TRUE) ;    // increase sail
      MoveWindow(hwndButton[3], 450-xScrollPos, 2-yScrollPos, 102, 22, TRUE) ;    // decrease sail
      MoveWindow(hwndButton[4], 333-xScrollPos, 557-yScrollPos, 105, 22, TRUE) ;  // helmsman
      MoveWindow(hwndButton[5], 333-xScrollPos, 605-yScrollPos, 105, 22, TRUE) ;  // yardsmen
      MoveWindow(hwndButton[6], 453-xScrollPos, 620-yScrollPos, 141, 29, TRUE) ;  // pause/restart
      break ;
    case 3: // 1152*864 pixels
      cxMap = 317 ;
      cxSternView = 101 ;
      cySternView = 206 ;
      MoveWindow (hwndChild[0], 7-xScrollPos, 29-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ; // plan view
      MoveWindow (hwndChild[1], 388-xScrollPos, 29-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ;  // sail plan
      MoveWindow (hwndChild[2], 770-xScrollPos, 29-yScrollPos, cxMap, cxMap, TRUE) ; // map
      MoveWindow (hwndChild[3], 201-xScrollPos, 396-yScrollPos, cxSternView, cySternView, TRUE) ;  // stern view
      MoveWindow (hwndText, 174-xScrollPos, 6-yScrollPos, 30 , 20, TRUE) ; // planview bearing
      MoveWindow(hwndSlider[0], 838-xScrollPos, 467-yScrollPos, 264, 21, TRUE) ; // fore mast
      MoveWindow(hwndSlider[1], 838-xScrollPos, 520-yScrollPos, 264, 21, TRUE) ; // main mast
      if (bBrig)
        {
        MoveWindow(hwndSlider[2], 0, 0, 0, 0, TRUE) ; // mizzen yards
        MoveWindow(hwndSlider[3], 838-xScrollPos, 573-yScrollPos, 264, 21, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 838-xScrollPos, 626-yScrollPos, 264, 21, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 680-xScrollPos, 684-yScrollPos, 136, 21, TRUE) ; // mainmast staysails
        MoveWindow(hwndButton[7], 0, 0, 0, 0, TRUE) ;                              // smaller jib
        MoveWindow(hwndButton[8], 0, 0, 0, 0, TRUE) ;                              // larger jib
        MoveWindow(hwndCheck[1], 0, 0, 0, 0, TRUE) ;                               // couple mizzen
        }
      else
        {
        MoveWindow(hwndSlider[2], 838-xScrollPos, 573-yScrollPos, 264, 21, TRUE) ; // mizzen mast
        MoveWindow(hwndSlider[3], 838-xScrollPos, 626-yScrollPos, 264, 21, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 838-xScrollPos, 679-yScrollPos, 264, 21, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 0, 0, 0, 0, TRUE) ;                              // mainmast staysails
        MoveWindow(hwndButton[7], 701-xScrollPos, 670-yScrollPos, 87, 21, TRUE) ;  // smaller jib
        MoveWindow(hwndButton[8], 701-xScrollPos, 695-yScrollPos, 87, 21, TRUE) ;  // larger jib
        MoveWindow(hwndCheck[1], 890-xScrollPos, 549-yScrollPos, 16, 16, TRUE) ;      // couple mizzen
        }
      MoveWindow(hwndSlider[5], 680-xScrollPos, 636-yScrollPos, 136, 21, TRUE) ;  // foremast staysails
      MoveWindow(hwndCheck[0], 874-xScrollPos, 442-yScrollPos, 16, 16, TRUE) ;  // couple yards
      MoveWindow(hwndCheck[2], 774-xScrollPos, 354-yScrollPos, 16, 16, TRUE) ; // track ship
      MoveWindow(hwndButton[0], 948-xScrollPos, 2-yScrollPos, 87, 25, TRUE) ; // zoom in
      MoveWindow(hwndButton[1], 1037-xScrollPos, 2-yScrollPos, 86, 25, TRUE) ; // zoom out
      MoveWindow(hwndButton[2], 392-xScrollPos, 2-yScrollPos, 115, 25, TRUE) ; // increase sail
      MoveWindow(hwndButton[3], 506-xScrollPos, 2-yScrollPos, 115, 25, TRUE) ; // decrease sail
      MoveWindow(hwndButton[4], 374-xScrollPos, 626-yScrollPos, 118, 25, TRUE) ; // helmsman
      MoveWindow(hwndButton[5], 374-xScrollPos, 679-yScrollPos, 118, 25, TRUE) ; // yardsmen
      MoveWindow(hwndButton[6], 510-xScrollPos, 698-yScrollPos, 159, 33, TRUE) ; // pause/restart
      break ;
    case 4: // 1280*1024 pixels
      cxMap = 353 ;
      cxSternView = 113 ;
      cySternView = 229 ;
      MoveWindow (hwndChild[0], 8-xScrollPos, 33-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ; // plan view
      MoveWindow (hwndChild[1], 431-xScrollPos, 33-yScrollPos, cxChildPixels, cxChildPixels, TRUE) ;  // sail plan
      MoveWindow (hwndChild[2], 855-xScrollPos, 33-yScrollPos, cxMap, cxMap, TRUE) ; // map
      MoveWindow (hwndChild[3], 224-xScrollPos, 440-yScrollPos, cxSternView, cySternView, TRUE) ;  // stern view
      MoveWindow (hwndText, 197-xScrollPos, 10-yScrollPos, 30 , 20, TRUE) ; // planview bearing
      MoveWindow(hwndSlider[0], 931-xScrollPos, 518-yScrollPos, 294, 24, TRUE) ; // fore mast
      MoveWindow(hwndSlider[1], 931-xScrollPos, 578-yScrollPos, 294, 24, TRUE) ; // main mast
      if (bBrig)
        {
        MoveWindow(hwndSlider[2], 0, 0, 0, 0, TRUE) ; // mizzen yards
        MoveWindow(hwndSlider[3], 931-xScrollPos, 637-yScrollPos, 294, 24, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 931-xScrollPos, 696-yScrollPos, 294, 24, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 755-xScrollPos, 760-yScrollPos, 151, 24, TRUE) ; // mainmast staysails
        MoveWindow(hwndButton[7], 0, 0, 0, 0, TRUE) ;                              // smaller jib
        MoveWindow(hwndButton[8], 0, 0, 0, 0, TRUE) ;                              // larger jib
        MoveWindow(hwndCheck[1], 0, 0, 0, 0, TRUE) ;                               // couple mizzen
        }
      else
        {
        MoveWindow(hwndSlider[2], 931-xScrollPos, 637-yScrollPos, 294, 24, TRUE) ; // mizzen mast
        MoveWindow(hwndSlider[3], 931-xScrollPos, 696-yScrollPos, 294, 24, TRUE) ; // spanker boom
        MoveWindow(hwndSlider[4], 931-xScrollPos, 755-yScrollPos, 294, 24, TRUE) ; // helm
        MoveWindow(hwndSlider[6], 0, 0, 0, 0, TRUE) ;                              // mainmast staysails
        MoveWindow(hwndButton[7], 778-xScrollPos, 747-yScrollPos, 97, 24, TRUE) ;  // smaller jib
        MoveWindow(hwndButton[8], 778-xScrollPos, 775-yScrollPos, 97, 24, TRUE) ;  // larger jib
        MoveWindow(hwndCheck[1], 989-xScrollPos, 610-yScrollPos, 16, 16, TRUE) ;      // couple mizzen
        }
      MoveWindow(hwndSlider[5], 755-xScrollPos, 707-yScrollPos, 151, 24, TRUE) ;  // foremast staysails
      MoveWindow(hwndCheck[0], 971-xScrollPos, 491-yScrollPos, 16, 16, TRUE) ;  // couple yards
      MoveWindow(hwndCheck[2], 860-xScrollPos, 394-yScrollPos, 16, 16, TRUE) ; // track ship
      MoveWindow(hwndButton[0], 1054-xScrollPos, 2-yScrollPos, 96, 28, TRUE) ; // zoom in
      MoveWindow(hwndButton[1], 1153-xScrollPos, 2-yScrollPos, 96, 28, TRUE) ; // zoom out
      MoveWindow(hwndButton[2], 435-xScrollPos, 2-yScrollPos, 128, 28, TRUE) ; // increase sail
      MoveWindow(hwndButton[3], 563-xScrollPos, 2-yScrollPos, 128, 28, TRUE) ; // decrease sail
      MoveWindow(hwndButton[4], 416-xScrollPos, 696-yScrollPos, 131, 28, TRUE) ; // helmsman
      MoveWindow(hwndButton[5], 416-xScrollPos, 757-yScrollPos, 131, 28, TRUE) ; // yardsmen
      MoveWindow(hwndButton[6], 566-xScrollPos, 775-yScrollPos, 176, 36, TRUE) ; // pause/restart
      break ;
    }

  bPlanViewIsChanged = TRUE ;
  bSailPlanIsChanged = TRUE ;
  bMapMustBePainted= TRUE ;
  bSternViewIsChanged = TRUE ;

 // legents for basic parameters
  yThis = yFirst ;
  TextOut (hdc, xLeft, yThis, "Latitude",8) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Longitude",9) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Heading",7) ; 
  TextOut (hdc, xColon, yThis, ":",1) ;
  TextOut (hdc, xValues, yThis, " ",2) ;
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Track",5) ; 
  TextOut (hdc, xColon, yThis, ":",1) ;
  TextOut (hdc, xValues, yThis, " ",2) ;
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Speed",5) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  TextOut (hdc, xValues, yThis, " kt",3) ;
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Leeway",6) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Rot (/min)",11) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Heel angle",10) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  yThis = yThis + cyTextHeight ;

 // legends for wind parameters
  yThis= yWind ; 
  SetTextColor(hdc, RGB(0,100,0)) ; // olive green
  TextOut (hdc, xLeft, yThis, "Wind", 4) ; 
  yThis = yThis + cyTextHeight ;
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "%3.0f ", fWindDirection)); 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, " at %3.0f kt (%d Bf)    ", fWindSpeed, Beaufort(fWindSpeed))); 
  yThis = yThis + cyTextHeight ;
  SetTextColor(hdc, RGB(255,0,0)) ;  // red
  TextOut (hdc, xLeft, yThis, "Apparent wind", 13) ; 
  yThis = yThis + cyTextHeight * 2 ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  TextOut (hdc, xLeft, yThis, "Relative wind", 13) ; 

 // legends for simulator speed
  yThis = yThis + 3*cyTextHeight ;
  TextOut (hdc, xLeft, yThis, "Sim. speed",10) ; 
  TextOut (hdc, xColon, yThis, ":",1) ; 
  TextOut (hdc, xColon+2, yThis, sMessage, sprintf(sMessage, "%2d", iAcceleration[iSpeedFactor])) ; 
 
 // message box
  NewFrame(hdc, xMessage-iSpacer, yMessage-iSpacer, cxMessage, cyMessage) ;
  if (iMessageNr > -1)
    {
    for (iI = 0 ; iI <= iMessageNr ; iI++)
      { 
      TextOut (hdc, xMessage, yMessage+ iI*cyTextHeight, sShipMessage[iI], strlen(sShipMessage[iI])) ; 
      } 
    } 

  // set up staysail display 
  NewFrame(hdc, xJib-iSpacer, yMessage-iSpacer, iJibWidth+2*iSpacer, cyMessage) ;
  if (bBrig)
    TextOut (hdc, xJib, yMessage+iSpacer, "Fore staysails:",15) ;
  else
    {
    switch (iJibSet)
      {
      case 0:
        TextOut (hdc, xJib, yMessage+iSpacer, "Large jib      :",16) ;
        SetWindowText(hwndButton[7], "Storm jib") ;
        SetWindowText(hwndButton[8], "Working jib") ;
        break ;
      case 1:
        TextOut (hdc, xJib, yMessage+iSpacer, "Working jib:    ",16) ;
        SetWindowText(hwndButton[7], "Storm jib") ;
        SetWindowText(hwndButton[8], "Large jib") ;
        break ;
      case 2:
        TextOut (hdc, xJib, yMessage+iSpacer, "Storm jib:      ",14) ;
        SetWindowText(hwndButton[7], "Working jib") ;
        SetWindowText(hwndButton[8], "Large jib") ;
      }
    if (iSailStatus[12]==4)
      {  
      EnableWindow (hwndButton[7], TRUE) ; //jib buttons enabled
      EnableWindow (hwndButton[8], TRUE) ;
      }
    else
      {
      EnableWindow (hwndButton[7], FALSE) ; //jib buttons disabled
      EnableWindow (hwndButton[8], FALSE) ;
      }
    }
  xJibValue = xJib + iJibWidth ;
  yJibValue = yMessage + iSpacer ;
  if (bBrig)
    TextOut (hdc, xJib, yMessage+4*iSpacer+cyTextHeight+cyScroll, "Main staysails:",15) ;
  yStaysValue = yMessage + 4*iSpacer + cyTextHeight + cyScroll ;

  // set up display of automatic helmsman and yardsmen
  NewFrame(hdc, xAuto-iSpacer, yMessage-iSpacer, cxAuto+2*iSpacer, cyMessage) ;
  SelectObject (hdc, GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xAuto, yMessage+29, xAuto+cxAuto, yMessage+29+cyTextHeight) ;
  SelectObject (hdc, GetStockObject(BLACK_PEN)) ;
  SetTextAlign (hdc,TA_CENTER|TA_TOP) ;
  TextOut (hdc, xAuto+cxAuto/2, yMessage+29, sHelmInstruction,strlen(sHelmInstruction)) ; 
  SelectObject (hdc, GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xAuto, yMessage+65, xAuto+cxAuto, yMessage+65+cyTextHeight) ;
  SelectObject (hdc, GetStockObject(BLACK_PEN)) ;
  if (bYardsmen)
    TextOut (hdc, xAuto+cxAuto/2, yMessage+65, " man braces ", 12) ; 
  else
    TextOut (hdc, xAuto+cxAuto/2, yMessage+65, "(await orders)",14) ; 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;

  // set up display of yard and helm sliders
  NewFrame(hdc, xSliders-iSpacer, yFirst, cxSliders, cySliders) ;
  SetTextColor(hdc, RGB(255,0,0)) ; // red
  TextOut (hdc, xSliders, yFirst+2*iSpacer, "PORT",4) ; 
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  SetTextColor(hdc, RGB(0,100,0)) ; // olive green
  TextOut (hdc, xSliders+cxSliders-2*iSpacer, yFirst+2*iSpacer, "STBD",4) ; 
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  yThis=yParameter+10 ; ;
  TextOut (hdc, xSelected, yThis, "Selected",8) ; 
  TextOut (hdc, xActual, yThis, "Actual",6) ; 
  if (bYardsmen)
    SetTextColor(hdc, RGB(160,160,160)) ; // gray
  yThis=ySliders[0] ;
  TextOut (hdc, xSliders, yThis, "Fore", 4) ; 
  yThis=ySliders[1] ;
  TextOut (hdc, xSliders, yThis, "Main", 4) ; 
  if (!bBrig)
    {
    yThis=ySliders[2] ;
    TextOut (hdc, xSliders, yThis, "Mizzen", 6) ; 
    }
  yThis=ySliders[3] ;
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
  TextOut (hdc, xSliders, yThis, "Spanker", 7) ; 
  yThis=ySliders[4] ;
  if (bHelmsman)
    SetTextColor(hdc, RGB(160,160,160)) ;  // gray
  if (bModernHelm)
    TextOut (hdc, xSliders, yThis, "Rudder", 6) ; 
  else
    TextOut (hdc, xSliders, yThis, "Helm      ", 10) ; 
  SetTextColor(hdc, RGB(0,0,0)) ;  // black
 
  // buttons 
  if (fMapWidth < 0.1)
    EnableWindow (hwndButton[0], FALSE) ;
  else
    EnableWindow (hwndButton[0], TRUE) ;
  if (fMapWidth > 20.)
    EnableWindow (hwndButton[1], FALSE) ;
  else
    EnableWindow (hwndButton[1], TRUE) ;
  if (iMadeSail == 15)
    EnableWindow (hwndButton[2], FALSE) ;
  else
    EnableWindow (hwndButton[2], TRUE) ;
  if (iMadeSail == 0)
    EnableWindow (hwndButton[3], FALSE) ;
  else
    EnableWindow (hwndButton[3], TRUE) ;

  TextOut (hdc, 558, 246, "Track ship",10) ; 

  // line for map width
  MoveToEx (hdc, 765, 20, NULL) ;
  LineTo (hdc, 776, 20) ;
  MoveToEx (hdc, 770, 20, NULL) ;
  LineTo (hdc, 770, 240) ;
  MoveToEx (hdc, 765, 240, NULL) ;
  LineTo (hdc, 776, 240) ;
  TextOut (hdc, 760, 113+cyTextHeight, "n.m.", 4) ; 
 
  // Set up display of sail parameters and totals
  NewFrame(hdc, xParameter, yFirst, iParameterWidth, iParameterHeight) ;
  NewFrame(hdc, xTotals-iSpacer, yMessage-iSpacer, iTotalsWidth, iTotalsHeight) ;
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xParameter+iSpacer, yFirst+iSpacer, xParameter+iParameterWidth-2*iSpacer, 
   yFirst+iParameterHeight-2*iSpacer) ;
  Rectangle (hdc, xTotals+iSpacer, yMessage+iSpacer, xTotals+iTotalsWidth-2*iSpacer, 
   yMessage+iTotalsHeight-2*iSpacer) ;
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  SelectSailParameter() ;

  // draw masts
  iI=iNrSails+1;
  if (bBrig)
    {
    MoveToEx (hdc, xSailParameter[0]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[0]-2*iOfsetTop, NULL );  
    LineTo (hdc, xSailParameter[0]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[iI]-2*iSpacer) ;  
    MoveToEx (hdc, xSailParameter[5]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[5]-2*iOfsetTop, NULL );  
    LineTo (hdc, xSailParameter[5]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[iI]-2*iSpacer) ;  
    }
  else
    {
    MoveToEx (hdc, xSailParameter[0]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[0]-2*iOfsetTop, NULL );  
    LineTo (hdc, xSailParameter[0]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[iI]-2*iSpacer) ;  
    MoveToEx (hdc, xSailParameter[4]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[4]-2*iOfsetTop, NULL );  
    LineTo (hdc, xSailParameter[4]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[iI]-2*iSpacer) ;  
    MoveToEx (hdc, xSailParameter[8]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[8]-2*iOfsetTop, NULL );  
    LineTo (hdc, xSailParameter[8]-(iOfsetLeft-iOfsetRight)/2, ySailParameter[iI]-2*iSpacer) ;  
    }

  // Draw hull
  MoveToEx (hdc, xParameter+3*iSpacer, ySailParameter[iI]-2*iSpacer, NULL) ;
  LineTo (hdc, xParameter+3*iSpacer, yFirst+iParameterHeight-3*iSpacer) ;  
  LineTo (hdc, xParameter+0.95*iParameterWidth, yFirst+iParameterHeight-3*iSpacer) ;  
  LineTo (hdc, xParameter+iParameterWidth-3*iSpacer, ySailParameter[iI]-2*iSpacer) ;  
  LineTo (hdc, xParameter+3*iSpacer, ySailParameter[iI]-2*iSpacer) ;  

   // force repainting
  strcpy(sOldHeading,"???") ;
  strcpy(sOldTrack,"???") ;
  strcpy(sOldSpeed,"???") ;
  strcpy(sOldLeewayAngle,"???") ;
  strcpy(sOldDegreesPerMin,"???") ;
  strcpy(sOldHeelingAngle,"???") ;
  ReleaseDC (hwnd, hdc) ;

  // display current values
  ShowYardShown() ;
  ShowMapWidth() ;
  if (bBrig)
    {
    ShowActualYardAngle(0) ;
    ShowActualYardAngle(1) ;
    ShowActualYardAngle(3) ;
    ShowSelectedYardAngle(0) ;
    ShowSelectedYardAngle(1) ;
    ShowSelectedYardAngle(3) ;
    ShowActualJibAngle() ;
    ShowActualStaysailsAngle() ;
    ShowActualHelmAngle() ;
    ShowSelectedHelmAngle() ;
    }
  else
    {
    ShowActualYardAngle(0) ;
    ShowActualYardAngle(1) ;
    ShowActualYardAngle(2) ;
    ShowActualYardAngle(3) ;
    ShowSelectedYardAngle(0) ;
    ShowSelectedYardAngle(1) ;
    ShowSelectedYardAngle(2) ;
    ShowSelectedYardAngle(3) ;
    ShowActualJibAngle() ;
    ShowActualHelmAngle() ;
    ShowSelectedHelmAngle() ;
    }
  }

/* --------------------------------------
   ShowParameters
   Version date = 03.08.13
----------------------------------------*/
void ShowParameters()
  {
  int   iI, iAngle ;
  float fAngle ;
  HDC   hdc ;
  char  cSuffix[5] ;
  int   xThis, yThis ;

  hdc = GetDC (hwnd) ;

  SelectObject (hdc, GetStockObject(NULL_PEN)) ;
  SetTextAlign (hdc, TA_RIGHT | TA_TOP) ;

  // Display the basic parameters
  xThis = xValues ;

  // position
  yThis = yFirst ;
  Rectangle (hdc, xThis, yThis, xThis+60, yThis + cyTextHeight-1) ;
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  TextOut (hdc, xThis, yThis, sMessage, sprintf(sMessage, "%5.3f", fabs(fLatitude))) ; 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  if (fLatitude >= 0.)
    TextOut (hdc, xThis, yThis, " nm N", 5) ; 
  else
    TextOut (hdc, xThis, yThis, " nm S", 5) ; 
  yThis = yFirst + cyTextHeight ;
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  Rectangle (hdc, xThis, yThis, xThis+60, yThis + cyTextHeight-1) ;
  TextOut (hdc, xThis, yThis, sMessage, sprintf(sMessage, "%5.3f", fabs(fLongitude))) ; 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  if (fLongitude >= 0.)
    TextOut (hdc, xThis, yThis, " nm E", 5) ; 
  else
    TextOut (hdc, xThis, yThis, " nm W", 5) ; 

  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  // heading
  yThis = yThis + cyTextHeight ;
  sprintf (sMessage, "%3.0f", fHeading) ; 
  if (strcmp(sMessage , sOldHeading) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis, yThis + cyTextHeight-1) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    strcpy(sOldHeading, sMessage) ;
    }

  // track
  yThis = yThis + cyTextHeight ;
  sprintf (sMessage, "%3.0f", fTrack) ; 
  if (strcmp(sMessage , sOldTrack) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis, yThis + cyTextHeight-1) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    strcpy(sOldTrack, sMessage) ;
    }

  // speed
  yThis = yThis + cyTextHeight ;
  sprintf (sMessage, "%5.1f", fSpeed) ; 
  if (strcmp(sMessage , sOldSpeed) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis, yThis + cyTextHeight-1) ;
    if (fSpeed < 0.0)
    SetTextColor(hdc, RGB(255,0,0)) ; // red
    TextOut (hdc, xThis, yThis, sMessage, 5) ; 
    SetTextColor(hdc, RGB(0,0,0)) ; // black
    strcpy(sOldSpeed, sMessage) ;
    }

  // leeway angle
  yThis = yThis + cyTextHeight ;
  sprintf (sMessage, "%3.0f", fabs(fLeewayAngle)) ; 
  if (strcmp(sMessage , sOldLeewayAngle) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis + 10 , yThis + cyTextHeight-1) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    if (Sgn((int) fLeewayAngle) > 0)
      TextOut (hdc, xThis, yThis, "  s ", 5) ; 
    else
      if (Sgn((int) fLeewayAngle) == 0)
        TextOut (hdc, xThis, yThis, "     ", 6) ; 
      else
        TextOut (hdc, xThis, yThis, "  p ", 5) ; 
    strcpy (sOldLeewayAngle, sMessage) ;
    } 

  // rotation
  yThis = yThis + cyTextHeight ;
  sprintf (sMessage, "%3.0f", fabs(fDegreesPerMin)) ; 
  if (strcmp(sMessage , sOldDegreesPerMin) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis + 10, yThis + cyTextHeight-1) ;
    SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    if (Sgn(fDegreesPerMin) > 0)
      TextOut (hdc, xThis, yThis, "  s ", 5) ; 
    else
      if (Sgn(fDegreesPerMin) == 0)
        TextOut (hdc, xThis, yThis, "     ", 6) ; 
      else
        TextOut (hdc, xThis, yThis, "  p ", 5) ; 
    strcpy (sOldDegreesPerMin, sMessage) ;
    }

  // heel angle
  yThis = yThis + cyTextHeight ;
  sprintf (sMessage, "%3.0f", fabs(fHeelingAngle)) ; 
  if (strcmp(sMessage , sOldHeelingAngle) != 0)
    {
    Rectangle (hdc, xThis - 20, yThis, xThis + 10, yThis + cyTextHeight-1) ;
    SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
    TextOut (hdc, xThis, yThis, sMessage, 3) ; 
    SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
    if (Sgn(fHeelingAngle) > 0)
      TextOut (hdc, xThis, yThis, "  s ", 5) ; 
    else
      if (Sgn(fHeelingAngle) == 0)
        TextOut (hdc, xThis, yThis, "     ", 6) ; 
      else
        TextOut (hdc, xThis, yThis, "  p ", 5) ;
    strcpy (sOldHeelingAngle, sMessage) ;
    }

  // wind parameters
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  yThis = yWind + 3 * cyTextHeight ;
  Rectangle (hdc, xWind - 30, yThis, xWind , yThis + cyTextHeight-1) ;
  SetTextColor(hdc, RGB(255,0,0)) ; // red
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "%3.0f  ", fAppWindDirection)); 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "at %3.0f kt ", fAppWindSpeed)); 
  yThis = yThis + 2 * cyTextHeight ;
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  SetTextAlign(hdc, TA_RIGHT | TA_TOP) ;
  Rectangle (hdc, xWind-30, yThis, xWind+45, yThis + cyTextHeight-1) ;
  TextOut (hdc, xWind, yThis, sMessage, sprintf(sMessage, "%3.0f  ", 180-fabs(fAppWindAngle))); 
  SetTextAlign(hdc, TA_LEFT | TA_TOP) ;
  if (fAppWindAngle > 0.98)
    TextOut (hdc, xWind, yThis, "starboard",9); 
  else
    if (fAppWindAngle < -0.98)
      TextOut (hdc, xWind, yThis, "port",4); 
    else
      TextOut (hdc, xWind, yThis, "astern",6); 

  // sail parameters
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  for (iI=0 ; iI <= iLastValueToShow ; iI++)
    {
    switch (iDisplayParam)
      {
      case 0 :
        sprintf (sMessage, "%7.0f", fabs(fSailAngle[iI])) ; 
        if (iI>=iSpanker)
        // fore-and-aft sails
        {
          switch (Sgn(fSailAngle[iI]))
            {
            case 1 :
              strcpy (cSuffix,  " p") ;
              break ;
            case 0 :
              strcpy (cSuffix,  "  ") ;
              break ;
            case -1 :
              strcpy (cSuffix,  " s") ;
            }
          }
        else
        // square sails
        {
          switch (Sgn(fSailAngle[iI]))
            {
            case 1 :
              strcpy (cSuffix,  " s") ;
              break ;
            case 0 :
              strcpy (cSuffix,  "  ") ;
              break ;
            case -1 :
              strcpy (cSuffix,  " p") ;
            }
          }
        break ;
      case 1  :
        if (iI >= iSpanker)
          // fore=and-aft sails
          fAngle = fabs(-fSailAngle[iI] - fAppWindAngle) ;
        else 
          fAngle = fabs((90.0 - fSailAngle[iI]) - fAppWindAngle) ;
        if (fAngle > 90)
          fAngle = 180 - fAngle ;
        if (iSailStatus[iI] == 6)
          fAngle = 0.0 ;
        sprintf (sMessage, "%7.0f", fAngle) ; 
        strcpy (cSuffix,  "  ") ;
        break ;
      case 2 :
        sprintf (sMessage, "%7.0f", fabs(fSailExposedFract[iI] * 100.0)) ; 
        strcpy (cSuffix,  " % ") ;
        break ;
      case 3 :
        sprintf (sMessage, "%7.0f", fabs(fSailPerpArea[iI])) ; 
        strcpy (cSuffix,  "   ") ;
        break ;
      case 4 :
        sprintf (sMessage, "%7.0f", fabs(fSailPerpFract[iI] * 100.0)) ; 
        strcpy (cSuffix,  " % ") ;
        break ;
      case 5 :
        sprintf (sMessage, "%7.0f", fabs(fSailPerpForce[iI])) ; 
        strcpy (cSuffix,  "   ") ;
        break ;
      case 6 :
        sprintf (sMessage, "%7.0f", fabs(fSailHeadwayForce[iI])) ; 
        switch (Sgn(fSailHeadwayForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " f ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " a ") ;
           }
         break ;
      case 7 :
        sprintf (sMessage, "%7.0f", fabs(fSailLeewayForce[iI])) ; 
        switch (Sgn(fSailLeewayForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " s ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " p ") ;
           }
         break ;
      case 8 :
        sprintf (sMessage, "%7.0f", fabs(fRotatForce[iI])) ; 
        switch (Sgn(fRotatForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " s ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " p ") ;
           }
         break ;
      case 9 :
        sprintf (sMessage, "%7.0f", fabs(fSailHeelForce[iI])) ; 
        switch (Sgn(fSailHeelForce[iI]))
          {
          case 1 :
            strcpy (cSuffix,  " s ") ;
            break ;
          case 0 :
            strcpy (cSuffix,  "   ") ;
            break ;
          case -1 :
            strcpy (cSuffix,  " p ") ;
           }
         break ;
       }
    //if (strcmp(sMessage , &sSailLastValue[iI]) != 0)
      {
      xThis = xSailParameter[iI] ;
      yThis = ySailParameter[iI] ;
      switch (iSailWarning[iI])
        {
        case 0 :
          SetTextColor(hdc, RGB(0,0,0)) ; // black
          break ;
        case 1 :
          SetTextColor(hdc, RGB(127,127,127)) ; // grey
          break ;
        case 2 :
          SetTextColor(hdc, RGB(163,0,0)) ; // dark red
          break ;
        case 3 :
          SetTextColor(hdc, RGB(255,0,0)) ; // red
          break ;
        case 4 :
          SetTextColor(hdc, RGB(255,255,255)) ; // white
          break ;
        }
      if (iI>iNrSails)
        SelectObject(hdc,GetStockObject(NULL_PEN)) ;
      Rectangle (hdc, xThis-iOfsetLeft, yThis-iOfsetTop, xThis+iOfsetRight, yThis+iOfsetBottom) ;
      SetTextAlign (hdc, TA_RIGHT | TA_TOP) ;
      TextOut (hdc, xThis, yThis, sMessage, 7); 
      SetTextAlign (hdc, TA_LEFT | TA_TOP) ;
      TextOut (hdc, xThis, yThis, cSuffix, 3); 
      strcpy (&sSailLastValue[iI], sMessage) ; 
      }     
    }

  // Lee or weather helm?
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xSliders+2, ySliders[4]+cyScroll+17, xSliders+25, ySliders[4]+cyScroll+cyTextHeight+18) ;
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  if (iHelmAngle != 0)
    {
    if (Sgn((float)iHelmAngle) == Sgn(fRelWindAngle))
      TextOut (hdc, xSliders+2, ySliders[4]+cyScroll+17, "W ",2); 
    else
      TextOut (hdc, xSliders+2, ySliders[4]+cyScroll+17, "L ",2); 
    }

  SetTextColor(hdc, RGB(0,0,0)) ; // black
  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;

  // Alerts
  SelectObject(hdc,GetStockObject(NULL_PEN)) ;
  Rectangle (hdc, xWind, yBottomLine, xWind+45, yBottomLine + cyTextHeight) ;
  sMessage[0]= '\0' ;
  iI = 0 ;
  if (bAlertSpeed)
    {
    sMessage[0]='S' ; 
    iI = 1 ;
    }
  if (bAlertLeeway)
    {
    sMessage[iI] = 'L' ; 
    iI++ ;
    }
  if (bAlertHeel)
    {
    sMessage[iI] = 'H' ; 
    iI++ ;
    }
  if (bAlertRotat)
    {
    sMessage[iI] = 'R' ; 
    iI++ ;
    }
  if (iI>0)
    {
    SetTextColor(hdc, RGB(255,0,0)) ; // red
    TextOut (hdc, xWind, yBottomLine, sMessage, iI); 
    SetTextColor(hdc, RGB(0,0,0)) ; // black
    }

  SelectObject(hdc,GetStockObject(BLACK_PEN)) ;
  SetTextColor(hdc, RGB(0,0,0)) ; // black
  ReleaseDC (hwnd, hdc) ;
  }
