/*-----------------------------------------
   STERNVIEW.C -- Plan view procedures module
   Version date: 99.09.19
  -----------------------------------------*/

#define   PI       3.1415927
#define   DEGTORAD PI/180.0

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "external.h"

// Prototypes:
void     SternView () ;

extern   int Sgn(float) ;

/*-----------------------------------------
   SternView: Update stern view    
   Version date: 99.09.19
-----------------------------------------*/

void SternView() 
  {
  HDC   hdc ;
  float fDistance ;
  POINT pDrawStern[4] ;
  int   iI ;
  float fRadians ;
  float fX, fX2, fX3, fY ;

  bSternViewIsChanged = FALSE ; //set to True by events requiring redraw

  hdc = GetDC (hwndChild[3]) ;
  SetMapMode (hdc, MM_ISOTROPIC) ;
  SetWindowExtEx (hdc, 300, 1200, NULL) ;
  SetViewportExtEx (hdc, 70, -143, NULL) ;
  SetViewportOrgEx (hdc, 33, 120, NULL) ;
  SelectObject (hdc, hBitmap) ;
  SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,RGB(0,56,207))) ; //sea blue
  SelectObject(hdc, hSeaBrush) ;
  Rectangle (hdc, -280, 0, 280, -150) ;
  DeleteObject(SelectObject(hdc,(HPEN) CreatePen(PS_SOLID,0,RGB(0,215,255)))) ; //sky blue
  SelectObject(hdc, hSkyBrush) ;
  Rectangle (hdc, -280, 1100, 280, 0) ;
  for (iI=0 ; iI <=3 ; iI++)
    {
    fRadians = fHeelingAngle * DEGTORAD ;
    fX=pStern[iI].y * 0.9 * fScale * sin(fRadians);
    fY=pStern[iI].y * 0.9 * fScale * cos(fRadians);
    fRadians = (fHeelingAngle+90) * DEGTORAD ;
    pDrawStern[iI].y=fY+(pStern[iI].x * 0.9 * fScale * cos(fRadians)) ;
    pDrawStern[iI].x=fX+(pStern[iI].x * 0.9 * fScale * sin(fRadians)) ;
    }
  fRadians = fHeelingAngle * DEGTORAD ;
  fDistance = 0.9 * fScale * (fSailHead[4]+2.0) ;
  fX = fDistance * sin (fRadians) ;
  fY = fDistance * cos (fRadians) ;

  SelectObject (hdc, (HPEN) CreatePen(PS_SOLID, 15, RGB(0,0,0))) ;
  SelectObject (hdc, hDeckBrush) ;
  MoveToEx (hdc,0,0,NULL) ;
  LineTo (hdc,fX,fY) ;
  BeginPath (hdc) ;
  DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN))) ; 
  PolyBezier (hdc,pDrawStern,4) ;
  CloseFigure (hdc) ;
  EndPath (hdc) ;
  StrokeAndFillPath (hdc) ;

  // draw centre of effort (if not all yards are square and wind<>0)
  if ((fabs(fHeelingAngle) < 50 ) && (fWindSpeed > 0))
    {
    if ((fCOEHeight < fSailHead[4]) && (fCOEHeight > fGunDeckHeight))
      {
      fDistance = 0.9 * fScale * fCOEHeight ;
      fY = cos(fRadians) * fDistance ;
      fX = sin(fRadians) * fDistance ;
      if (Sgn(fHeelingAngle) > 0)
        {
        fX = fX - 50 ;
        fX2 = fX - 150 ;
        fX3 = fX - 50 ;
        }
      else
        {
        fX = fX + 50 ;
        fX2 = fX + 150 ;
        fX3 = fX + 50 ;
        }
      SelectObject(hdc, (HPEN) CreatePen(PS_SOLID,0,RGB(255,0,0))) ; //red
      MoveToEx (hdc, fX, fY, NULL) ;
      LineTo (hdc, fX2, fY) ;
      MoveToEx (hdc, fX, fY, NULL) ;
      LineTo (hdc, fX3, fY+50) ;
      MoveToEx (hdc, fX, fY, NULL) ;
      LineTo (hdc, fX3, fY-50) ;
      DeleteObject(SelectObject(hdc,GetStockObject(BLACK_PEN))) ;
      }
    }
  ReleaseDC (hwndChild[3],hdc) ;
  }


